# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################
import cgi
import cPickle
import logging
import os
import shutil
import zipfile
import json
from tempfile import mkstemp
from datetime import datetime, timedelta

log = logging.getLogger(__name__)

from PyQt4 import QtCore, QtGui

from openlp.core.lib import OpenLPToolbar, ServiceItem, Receiver, build_icon, \
    ItemCapabilities, SettingsManager, translate, str_to_bool, \
    check_directory_exists
from openlp.core.lib.theme import ThemeLevel
from openlp.core.lib.settings import Settings
from openlp.core.lib.ui import UiStrings, critical_error_message_box, \
    create_widget_action, find_and_set_in_combo_box
from openlp.core.ui import ServiceNoteForm, ServiceItemEditForm, StartTimeForm
from openlp.core.ui.printserviceform import PrintServiceForm
from openlp.core.utils import AppLocation, delete_file, split_filename, \
    format_time
from openlp.core.utils.actions import ActionList, CategoryOrder

class ServiceManagerList(QtGui.QTreeWidget):
    """
    Set up key bindings and mouse behaviour for the service list
    """
    def __init__(self, serviceManager, parent=None):
        QtGui.QTreeWidget.__init__(self, parent)
        self.serviceManager = serviceManager

    def keyPressEvent(self, event):
        if isinstance(event, QtGui.QKeyEvent):
            # here accept the event and do something
            if event.key() == QtCore.Qt.Key_Up:
                self.serviceManager.onMoveSelectionUp()
                event.accept()
            elif event.key() == QtCore.Qt.Key_Down:
                self.serviceManager.onMoveSelectionDown()
                event.accept()
            elif event.key() == QtCore.Qt.Key_Delete:
                self.serviceManager.onDeleteFromService()
                event.accept()
            event.ignore()
        else:
            event.ignore()

    def mouseMoveEvent(self, event):
        """
        Drag and drop event does not care what data is selected
        as the recipient will use events to request the data move
        just tell it what plugin to call
        """
        if event.buttons() != QtCore.Qt.LeftButton:
            event.ignore()
            return
        if not self.itemAt(self.mapFromGlobal(QtGui.QCursor.pos())):
            event.ignore()
            return
        drag = QtGui.QDrag(self)
        mimeData = QtCore.QMimeData()
        drag.setMimeData(mimeData)
        mimeData.setText(u'ServiceManager')
        drag.start(QtCore.Qt.CopyAction)


class ServiceManager(QtGui.QWidget):
    """
    Manages the services. This involves taking text strings from plugins and
    adding them to the service. This service can then be zipped up with all
    the resources used into one OSZ file for use on any OpenLP v2 installation.
    Also handles the UI tasks of moving things up and down etc.
    """
    def __init__(self, mainwindow, parent=None):
        """
        Sets up the service manager, toolbars, list view, et al.
        """
        QtGui.QWidget.__init__(self, parent)
        self.mainwindow = mainwindow
        self.serviceItems = []
        self.suffixes = []
        self.dropPosition = 0
        self.expandTabs = False
        self.serviceId = 0
        # is a new service and has not been saved
        self._modified = False
        self._fileName = u''
        self.serviceNoteForm = ServiceNoteForm(self.mainwindow)
        self.serviceItemEditForm = ServiceItemEditForm(self.mainwindow)
        self.startTimeForm = StartTimeForm(self.mainwindow)
        # start with the layout
        self.layout = QtGui.QVBoxLayout(self)
        self.layout.setSpacing(0)
        self.layout.setMargin(0)
        # Create the top toolbar
        self.toolbar = OpenLPToolbar(self)
        self.toolbar.addToolbarAction(u'newService',
            text=UiStrings().NewService, icon=u':/general/general_new.png',
            tooltip=UiStrings().CreateService,
            triggers=self.onNewServiceClicked)
        self.toolbar.addToolbarAction(u'openService',
            text=UiStrings().OpenService, icon=u':/general/general_open.png',
            tooltip=translate('OpenLP.ServiceManager',
            'Load an existing service.'), triggers=self.onLoadServiceClicked)
        self.toolbar.addToolbarAction(u'saveService',
            text=UiStrings().SaveService, icon=u':/general/general_save.png',
            tooltip=translate('OpenLP.ServiceManager', 'Save this service.'),
            triggers=self.saveFile)
        self.toolbar.addSeparator()
        self.themeLabel = QtGui.QLabel(u'%s:' % UiStrings().Theme, self)
        self.themeLabel.setMargin(3)
        self.themeLabel.setObjectName(u'themeLabel')
        self.toolbar.addToolbarWidget(self.themeLabel)
        self.themeComboBox = QtGui.QComboBox(self.toolbar)
        self.themeComboBox.setToolTip(translate('OpenLP.ServiceManager',
            'Select a theme for the service.'))
        self.themeComboBox.setSizeAdjustPolicy(
            QtGui.QComboBox.AdjustToMinimumContentsLength)
        self.themeComboBox.setSizePolicy(
            QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        self.themeComboBox.setObjectName(u'themeComboBox')
        self.toolbar.addToolbarWidget(self.themeComboBox)
        self.toolbar.setObjectName(u'toolbar')
        self.layout.addWidget(self.toolbar)
        # Create the service manager list
        self.serviceManagerList = ServiceManagerList(self)
        self.serviceManagerList.setEditTriggers(
            QtGui.QAbstractItemView.CurrentChanged |
            QtGui.QAbstractItemView.DoubleClicked |
            QtGui.QAbstractItemView.EditKeyPressed)
        self.serviceManagerList.setDragDropMode(
            QtGui.QAbstractItemView.DragDrop)
        self.serviceManagerList.setAlternatingRowColors(True)
        self.serviceManagerList.setHeaderHidden(True)
        self.serviceManagerList.setExpandsOnDoubleClick(False)
        self.serviceManagerList.setContextMenuPolicy(
            QtCore.Qt.CustomContextMenu)
        QtCore.QObject.connect(self.serviceManagerList,
            QtCore.SIGNAL('customContextMenuRequested(QPoint)'),
            self.contextMenu)
        self.serviceManagerList.setObjectName(u'serviceManagerList')
        # enable drop
        self.serviceManagerList.__class__.dragEnterEvent = self.dragEnterEvent
        self.serviceManagerList.__class__.dragMoveEvent = self.dragEnterEvent
        self.serviceManagerList.__class__.dropEvent = self.dropEvent
        self.layout.addWidget(self.serviceManagerList)
        # Add the bottom toolbar
        self.orderToolbar = OpenLPToolbar(self)
        action_list = ActionList.get_instance()
        action_list.add_category(
            unicode(UiStrings().Service), CategoryOrder.standardToolbar)
        self.serviceManagerList.moveTop = self.orderToolbar.addToolbarAction(
            u'moveTop', text=translate('OpenLP.ServiceManager', 'Move to &top'),
            icon=u':/services/service_top.png', tooltip=translate(
            'OpenLP.ServiceManager', 'Move item to the top of the service.'),
            shortcuts=[QtCore.Qt.Key_Home], category=UiStrings().Service,
            triggers=self.onServiceTop)
        self.serviceManagerList.moveUp = self.orderToolbar.addToolbarAction(
            u'moveUp', text=translate('OpenLP.ServiceManager', 'Move &up'),
            icon=u':/services/service_up.png',
            tooltip=translate('OpenLP.ServiceManager',
            'Move item up one position in the service.'),
            shortcuts=[QtCore.Qt.Key_PageUp], category=UiStrings().Service,
            triggers=self.onServiceUp)
        self.serviceManagerList.moveDown = self.orderToolbar.addToolbarAction(
            u'moveDown', text=translate('OpenLP.ServiceManager', 'Move &down'),
            icon=u':/services/service_down.png',
            tooltip=translate('OpenLP.ServiceManager',
            'Move item down one position in the service.'),
            shortcuts=[QtCore.Qt.Key_PageDown], category=UiStrings().Service,
            triggers=self.onServiceDown)
        self.serviceManagerList.moveBottom = self.orderToolbar.addToolbarAction(
            u'moveBottom',
            text=translate('OpenLP.ServiceManager', 'Move to &bottom'),
            icon=u':/services/service_bottom.png', tooltip=translate(
            'OpenLP.ServiceManager', 'Move item to the end of the service.'),
            shortcuts=[QtCore.Qt.Key_End], category=UiStrings().Service,
            triggers=self.onServiceEnd)
        self.serviceManagerList.down = self.orderToolbar.addToolbarAction(
            u'down', text=translate('OpenLP.ServiceManager', 'Move &down'),
            tooltip=translate('OpenLP.ServiceManager',
            'Moves the selection down the window.'), visible=False,
            shortcuts=[QtCore.Qt.Key_Down], triggers=self.onMoveSelectionDown)
        action_list.add_action(self.serviceManagerList.down)
        self.serviceManagerList.up = self.orderToolbar.addToolbarAction(
            u'up', text=translate('OpenLP.ServiceManager', 'Move up'),
            tooltip=translate('OpenLP.ServiceManager',
            'Moves the selection up the window.'), visible=False,
            shortcuts=[QtCore.Qt.Key_Up], triggers=self.onMoveSelectionUp)
        action_list.add_action(self.serviceManagerList.up)
        self.orderToolbar.addSeparator()
        self.serviceManagerList.delete = self.orderToolbar.addToolbarAction(
            u'delete',
            text=translate('OpenLP.ServiceManager', '&Delete From Service'),
            icon=u':/general/general_delete.png',
            tooltip=translate('OpenLP.ServiceManager',
            'Delete the selected item from the service.'),
            shortcuts=[QtCore.Qt.Key_Delete],
            triggers=self.onDeleteFromService)
        self.orderToolbar.addSeparator()
        self.serviceManagerList.expand = self.orderToolbar.addToolbarAction(
            u'expand', text=translate('OpenLP.ServiceManager', '&Expand all'),
            icon=u':/services/service_expand_all.png', tooltip=translate(
            'OpenLP.ServiceManager', 'Expand all the service items.'),
            shortcuts=[QtCore.Qt.Key_Plus], category=UiStrings().Service,
            triggers=self.onExpandAll)
        self.serviceManagerList.collapse = self.orderToolbar.addToolbarAction(
            u'collapse',
            text=translate('OpenLP.ServiceManager', '&Collapse all'),
            icon=u':/services/service_collapse_all.png', tooltip=translate(
            'OpenLP.ServiceManager', 'Collapse all the service items.'),
            shortcuts=[QtCore.Qt.Key_Minus], category=UiStrings().Service,
            triggers=self.onCollapseAll)
        self.orderToolbar.addSeparator()
        self.serviceManagerList.makeLive = self.orderToolbar.addToolbarAction(
            u'makeLive', text=translate('OpenLP.ServiceManager', 'Go Live'),
            icon=u':/general/general_live.png', tooltip=translate(
            'OpenLP.ServiceManager', 'Send the selected item to Live.'),
            shortcuts=[QtCore.Qt.Key_Enter, QtCore.Qt.Key_Return],
            category=UiStrings().Service, triggers=self.makeLive)
        self.layout.addWidget(self.orderToolbar)
        # Connect up our signals and slots
        QtCore.QObject.connect(self.themeComboBox,
            QtCore.SIGNAL(u'activated(int)'), self.onThemeComboBoxSelected)
        QtCore.QObject.connect(self.serviceManagerList,
            QtCore.SIGNAL(u'doubleClicked(QModelIndex)'), self.onMakeLive)
        QtCore.QObject.connect(self.serviceManagerList,
           QtCore.SIGNAL(u'itemCollapsed(QTreeWidgetItem*)'), self.collapsed)
        QtCore.QObject.connect(self.serviceManagerList,
           QtCore.SIGNAL(u'itemExpanded(QTreeWidgetItem*)'), self.expanded)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'theme_update_list'), self.updateThemeList)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'servicemanager_preview_live'), self.previewLive)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'servicemanager_next_item'), self.nextItem)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'servicemanager_previous_item'), self.previousItem)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'servicemanager_set_item'), self.onSetItem)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'config_updated'), self.configUpdated)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'config_screen_changed'),
            self.regenerateServiceItems)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'theme_update_global'), self.themeChange)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'service_item_update'), self.serviceItemUpdate)
        # Last little bits of setting up
        self.service_theme = unicode(Settings().value(
            self.mainwindow.serviceManagerSettingsSection + u'/service theme',
            QtCore.QVariant(u'')).toString())
        self.servicePath = AppLocation.get_section_data_path(u'servicemanager')
        # build the drag and drop context menu
        self.dndMenu = QtGui.QMenu()
        self.newAction = self.dndMenu.addAction(
            translate('OpenLP.ServiceManager', '&Add New Item'))
        self.newAction.setIcon(build_icon(u':/general/general_edit.png'))
        self.addToAction = self.dndMenu.addAction(
            translate('OpenLP.ServiceManager', '&Add to Selected Item'))
        self.addToAction.setIcon(build_icon(u':/general/general_edit.png'))
        # build the context menu
        self.menu = QtGui.QMenu()
        self.editAction = create_widget_action(self.menu,
            text=translate('OpenLP.ServiceManager', '&Edit Item'),
            icon=u':/general/general_edit.png', triggers=self.remoteEdit)
        self.maintainAction = create_widget_action(self.menu,
            text=translate('OpenLP.ServiceManager', '&Reorder Item'),
            icon=u':/general/general_edit.png',
            triggers=self.onServiceItemEditForm)
        self.notesAction = create_widget_action(self.menu,
            text=translate('OpenLP.ServiceManager', '&Notes'),
            icon=u':/services/service_notes.png',
            triggers=self.onServiceItemNoteForm)
        self.timeAction = create_widget_action(self.menu,
            text=translate('OpenLP.ServiceManager', '&Start Time'),
            icon=u':/media/media_time.png', triggers=self.onStartTimeForm)
        # Add already existing delete action to the menu.
        self.menu.addAction(self.serviceManagerList.delete)
        self.menu.addSeparator()
        self.previewAction = create_widget_action(self.menu,
            text=translate('OpenLP.ServiceManager', 'Show &Preview'),
            icon=u':/general/general_preview.png', triggers=self.makePreview)
        # Add already existing make live action to the menu.
        self.menu.addAction(self.serviceManagerList.makeLive)
        self.menu.addSeparator()
        self.themeMenu = QtGui.QMenu(
            translate('OpenLP.ServiceManager', '&Change Item Theme'))
        self.menu.addMenu(self.themeMenu)
        self.serviceManagerList.addActions(
            [self.serviceManagerList.moveDown,
            self.serviceManagerList.moveUp,
            self.serviceManagerList.makeLive,
            self.serviceManagerList.moveTop,
            self.serviceManagerList.moveBottom,
            self.serviceManagerList.up,
            self.serviceManagerList.down,
            self.serviceManagerList.expand,
            self.serviceManagerList.collapse
            ])
        self.configUpdated()

    def setModified(self, modified=True):
        """
        Setter for property "modified". Sets whether or not the current service
        has been modified.
        """
        if modified:
            self.serviceId += 1
        self._modified = modified
        serviceFile = self.shortFileName() or translate(
            'OpenLP.ServiceManager', 'Untitled Service')
        self.mainwindow.setServiceModified(modified, serviceFile)

    def isModified(self):
        """
        Getter for boolean property "modified".
        """
        return self._modified

    def setFileName(self, fileName):
        """
        Setter for service file.
        """
        self._fileName = unicode(fileName)
        self.mainwindow.setServiceModified(self.isModified(),
            self.shortFileName())
        Settings(). \
            setValue(u'servicemanager/last file',QtCore.QVariant(fileName))

    def fileName(self):
        """
        Return the current file name including path.
        """
        return self._fileName

    def shortFileName(self):
        """
        Return the current file name, excluding the path.
        """
        return split_filename(self._fileName)[1]

    def configUpdated(self):
        """
        Triggered when Config dialog is updated.
        """
        self.expandTabs = Settings().value(
            u'advanced/expand service item',
            QtCore.QVariant(u'False')).toBool()

    def supportedSuffixes(self, suffix):
        """
        Adds Suffixes supported to the master list.  Called from Plugins.

        ``suffix``
            New Suffix to be supported
        """
        self.suffixes.append(suffix)

    def onNewServiceClicked(self):
        """
        Create a new service.
        """
        if self.isModified():
            result = self.saveModifiedService()
            if result == QtGui.QMessageBox.Cancel:
                return False
            elif result == QtGui.QMessageBox.Save:
                if not self.saveFile():
                    return False
        self.newFile()

    def onLoadServiceClicked(self, loadFile=None):
        """
        Loads the service file and saves the existing one it there is one
        unchanged

        ``loadFile``
            The service file to the loaded.  Will be None is from menu so
            selection will be required.
        """
        if self.isModified():
            result = self.saveModifiedService()
            if result == QtGui.QMessageBox.Cancel:
                return False
            elif result == QtGui.QMessageBox.Save:
                self.saveFile()
        if not loadFile:
            fileName = unicode(QtGui.QFileDialog.getOpenFileName(
                self.mainwindow,
                translate('OpenLP.ServiceManager', 'Open File'),
                SettingsManager.get_last_dir(
                self.mainwindow.serviceManagerSettingsSection),
                translate('OpenLP.ServiceManager',
                'OpenLP Service Files (*.osz)')))
            if not fileName:
                return False
        else:
            fileName = loadFile
        SettingsManager.set_last_dir(
            self.mainwindow.serviceManagerSettingsSection,
            split_filename(fileName)[0])
        self.loadFile(fileName)

    def saveModifiedService(self):
        return QtGui.QMessageBox.question(self.mainwindow,
            translate('OpenLP.ServiceManager', 'Modified Service'),
            translate('OpenLP.ServiceManager', 'The current service has '
            'been modified. Would you like to save this service?'),
            QtGui.QMessageBox.Save | QtGui.QMessageBox.Discard |
            QtGui.QMessageBox.Cancel, QtGui.QMessageBox.Save)

    def onRecentServiceClicked(self):
        sender = self.sender()
        self.loadFile(sender.data().toString())

    def newFile(self):
        """
        Create a blank new service file.
        """
        self.serviceManagerList.clear()
        self.serviceItems = []
        self.setFileName(u'')
        self.serviceId += 1
        self.setModified(False)
        Settings(). \
            setValue(u'servicemanager/last file',QtCore.QVariant(u''))
        Receiver.send_message(u'servicemanager_new_service')

    def saveFile(self):
        """
        Save the current service file.

        A temporary file is created so that we don't overwrite the existing one
        and leave a mangled service file should there be an error when saving.
        Audio files are also copied into the service manager directory, and
        then packaged into the zip file.
        """
        if not self.fileName():
            return self.saveFileAs()
        temp_file, temp_file_name = mkstemp(u'.osz', u'openlp_')
        # We don't need the file handle.
        os.close(temp_file)
        log.debug(temp_file_name)
        path_file_name = unicode(self.fileName())
        path, file_name = os.path.split(path_file_name)
        basename = os.path.splitext(file_name)[0]
        service_file_name = '%s.osj' % basename
        log.debug(u'ServiceManager.saveFile - %s', path_file_name)
        SettingsManager.set_last_dir(
            self.mainwindow.serviceManagerSettingsSection,
            path)
        service = []
        write_list = []
        missing_list = []
        audio_files = []
        total_size = 0
        Receiver.send_message(u'cursor_busy')
        # Number of items + 1 to zip it
        self.mainwindow.displayProgressBar(len(self.serviceItems) + 1)
        # Get list of missing files, and list of files to write
        for item in self.serviceItems:
            if not item[u'service_item'].uses_file():
                continue
            for frame in item[u'service_item'].get_frames():
                path_from = item[u'service_item'].get_frame_path(frame=frame)
                if path_from in write_list or path_from in missing_list:
                    continue
                if not os.path.exists(path_from):
                    missing_list.append(path_from)
                else:
                    write_list.append(path_from)
        if missing_list:
            Receiver.send_message(u'cursor_normal')
            title = unicode(translate('OpenLP.ServiceManager',
                'Service File(s) Missing'))
            message = unicode(translate('OpenLP.ServiceManager',
                'The following file(s) in the service are missing:\n\t%s\n\n'
                'These files will be removed if you continue to save.')
                ) % "\n\t".join(missing_list)
            answer = QtGui.QMessageBox.critical(self, title,
                message,
                QtGui.QMessageBox.StandardButtons(
                    QtGui.QMessageBox.Ok | QtGui.QMessageBox.Cancel))
            if answer == QtGui.QMessageBox.Cancel:
                self.mainwindow.finishedProgressBar()
                return False
            Receiver.send_message(u'cursor_busy')
        # Check if item contains a missing file.
        for item in list(self.serviceItems):
            self.mainwindow.incrementProgressBar()
            item[u'service_item'].remove_invalid_frames(missing_list)
            if not item[u'service_item'].validate():
                self.serviceItems.remove(item)
            else:
                service_item = item[u'service_item'].get_service_repr()
                if service_item[u'header'][u'background_audio']:
                    for i, filename in enumerate(
                        service_item[u'header'][u'background_audio']):
                        new_file = os.path.join(u'audio',
                            item[u'service_item']._uuid, filename)
                        audio_files.append((filename, new_file))
                        service_item[u'header'][u'background_audio'][i] = \
                            new_file
                # Add the service item to the service.
                service.append({u'serviceitem': service_item})
        self.repaintServiceList(-1, -1)
        for file in write_list:
            file_size = os.path.getsize(file)
            total_size += file_size
        log.debug(u'ServiceManager.saveFile - ZIP contents size is %i bytes' %
            total_size)
        service_content = json.dumps(service)
        # Usual Zip file cannot exceed 2GiB, file with Zip64 cannot be
        # extracted using unzip in UNIX.
        allow_zip_64 = (total_size > 2147483648 + len(service_content))
        log.debug(u'ServiceManager.saveFile - allowZip64 is %s' % allow_zip_64)
        zip = None
        success = True
        self.mainwindow.incrementProgressBar()
        try:
            zip = zipfile.ZipFile(temp_file_name, 'w', zipfile.ZIP_STORED,
                allow_zip_64)
            # First we add service contents.
            # We save ALL filenames into ZIP using UTF-8.
            zip.writestr(service_file_name.encode(u'utf-8'), service_content)
            # Finally add all the listed media files.
            for write_from in write_list:
                zip.write(write_from, write_from.encode(u'utf-8'))
            for audio_from, audio_to in audio_files:
                if audio_from.startswith(u'audio'):
                    # When items are saved, they get new UUID's. Let's copy the
                    # file to the new location. Unused files can be ignored,
                    # OpenLP automatically cleans up the service manager dir on
                    # exit.
                    audio_from = os.path.join(self.servicePath, audio_from)
                save_file = os.path.join(self.servicePath, audio_to)
                save_path = os.path.split(save_file)[0]
                check_directory_exists(save_path)
                if not os.path.exists(save_file):
                    shutil.copy(audio_from, save_file)
                zip.write(audio_from, audio_to.encode(u'utf-8'))
        except IOError:
            log.exception(u'Failed to save service to disk: %s', temp_file_name)
            Receiver.send_message(u'openlp_error_message', {
                u'title': translate(u'OpenLP.ServiceManager',
                u'Error Saving File'),
                u'message': translate(u'OpenLP.ServiceManager',
                u'There was an error saving your file.')
            })
            success = False
        finally:
            if zip:
                zip.close()
        self.mainwindow.finishedProgressBar()
        Receiver.send_message(u'cursor_normal')
        if success:
            try:
                shutil.copy(temp_file_name, path_file_name)
            except:
                return self.saveFileAs()
            self.mainwindow.addRecentFile(path_file_name)
            self.setModified(False)
        delete_file(temp_file_name)
        return success

    def saveFileAs(self):
        """
        Get a file name and then call :func:`ServiceManager.saveFile` to
        save the file.
        """
        default_service_enabled = Settings().value(
            u'advanced/default service enabled', QtCore.QVariant(True)).toBool()
        if default_service_enabled:
            service_day = Settings().value(
                u'advanced/default service day', 7).toInt()[0]
            if service_day == 7:
                local_time = datetime.now()
            else:
                service_hour = Settings().value(
                    u'advanced/default service hour', 11).toInt()[0]
                service_minute = Settings().value(
                    u'advanced/default service minute', 0).toInt()[0]
                now = datetime.now()
                day_delta = service_day - now.weekday()
                if day_delta < 0:
                    day_delta += 7
                time = now + timedelta(days=day_delta)
                local_time = time.replace(hour=service_hour, minute=service_minute)
            default_pattern = unicode(Settings().value(
                u'advanced/default service name',
                translate('OpenLP.AdvancedTab', 'Service %Y-%m-%d %H-%M',
                    'This may not contain any of the following characters: '
                    '/\\?*|<>\[\]":+\nSee http://docs.python.org/library/'
                    'datetime.html#strftime-strptime-behavior for more '
                    'information.')).toString())
            default_filename = format_time(default_pattern, local_time)
        else:
            default_filename = u''
        directory = unicode(SettingsManager.get_last_dir(
            self.mainwindow.serviceManagerSettingsSection))
        path = os.path.join(directory, default_filename)
        fileName = unicode(QtGui.QFileDialog.getSaveFileName(self.mainwindow,
            UiStrings().SaveService, path,
            translate('OpenLP.ServiceManager', 'OpenLP Service Files (*.osz)')))
        if not fileName:
            return False
        if os.path.splitext(fileName)[1] == u'':
            fileName += u'.osz'
        else:
            ext = os.path.splitext(fileName)[1]
            fileName.replace(ext, u'.osz')
        self.setFileName(fileName)
        return self.saveFile()

    def loadFile(self, fileName):
        if not fileName:
            return False
        fileName = unicode(fileName)
        if not os.path.exists(fileName):
            return False
        zip = None
        fileTo = None
        try:
            zip = zipfile.ZipFile(fileName)
            for zipinfo in zip.infolist():
                try:
                    ucsfile = zipinfo.filename.decode(u'utf-8')
                except UnicodeDecodeError:
                    log.exception(u'Filename "%s" is not valid UTF-8' %
                        zipinfo.filename.decode(u'utf-8', u'replace'))
                    critical_error_message_box(
                        message=translate('OpenLP.ServiceManager',
                        'File is not a valid service.\n'
                        'The content encoding is not UTF-8.'))
                    continue
                osfile = ucsfile.replace(u'/', os.path.sep)
                if not osfile.startswith(u'audio'):
                    osfile = os.path.split(osfile)[1]
                log.debug(u'Extract file: %s', osfile)
                zipinfo.filename = osfile
                zip.extract(zipinfo, self.servicePath)
                if osfile.endswith(u'osd') or osfile.endswith(u'osj'):
                    p_file = os.path.join(self.servicePath, osfile)
            if 'p_file' in locals():
                Receiver.send_message(u'cursor_busy')
                fileTo = open(p_file, u'r')
                if p_file.endswith(u'osj'):
                    items = json.load(fileTo)
                else:
                    items = cPickle.load(fileTo)
                fileTo.close()
                self.newFile()
                self.mainwindow.displayProgressBar(len(items))
                for item in items:
                    self.mainwindow.incrementProgressBar()
                    serviceItem = ServiceItem()
                    serviceItem.renderer = self.mainwindow.renderer
                    serviceItem.set_from_service(item, self.servicePath)
                    self.validateItem(serviceItem)
                    self.load_item_uuid = 0
                    if serviceItem.is_capable(ItemCapabilities.OnLoadUpdate):
                        Receiver.send_message(u'%s_service_load' %
                            serviceItem.name.lower(), serviceItem)
                    # if the item has been processed
                    if serviceItem._uuid == self.load_item_uuid:
                        serviceItem.edit_id = int(self.load_item_edit_id)
                        serviceItem.temporary_edit = self.load_item_temporary
                    self.addServiceItem(serviceItem, repaint=False)
                delete_file(p_file)
                self.setFileName(fileName)
                self.mainwindow.addRecentFile(fileName)
                self.setModified(False)
                Settings().setValue(
                    'servicemanager/last file', QtCore.QVariant(fileName))
            else:
                critical_error_message_box(
                    message=translate('OpenLP.ServiceManager',
                    'File is not a valid service.'))
                log.exception(u'File contains no service data')
        except (IOError, NameError, zipfile.BadZipfile):
            log.exception(u'Problem loading service file %s' % fileName)
            critical_error_message_box(
                message=translate('OpenLP.ServiceManager',
                'File could not be opened because it is corrupt.'))
        except zipfile.BadZipfile:
            if os.path.getsize(fileName) == 0:
                log.exception(u'Service file is zero sized: %s' % fileName)
                QtGui.QMessageBox.information(self,
                    translate('OpenLP.ServiceManager', 'Empty File'),
                    translate('OpenLP.ServiceManager', 'This service file '
                    'does not contain any data.'))
            else:
                log.exception(u'Service file is cannot be extracted as zip: '
                    u'%s' % fileName)
                QtGui.QMessageBox.information(self,
                    translate('OpenLP.ServiceManager', 'Corrupt File'),
                    translate('OpenLP.ServiceManager', 'This file is either '
                    'corrupt or it is not an OpenLP 2.0 service file.'))
            return
        finally:
            if fileTo:
                fileTo.close()
            if zip:
                zip.close()
        self.mainwindow.finishedProgressBar()
        Receiver.send_message(u'cursor_normal')
        self.repaintServiceList(-1, -1)

    def loadLastFile(self):
        """
        Load the last service item from the service manager when the
        service was last closed. Can be blank if there was no service
        present.
        """
        fileName = Settings(). \
            value(u'servicemanager/last file',QtCore.QVariant(u'')).toString()
        if fileName:
            self.loadFile(fileName)

    def contextMenu(self, point):
        item = self.serviceManagerList.itemAt(point)
        if item is None:
            return
        if item.parent() is None:
            pos = item.data(0, QtCore.Qt.UserRole).toInt()[0]
        else:
            pos = item.parent().data(0, QtCore.Qt.UserRole).toInt()[0]
        serviceItem = self.serviceItems[pos - 1]
        self.editAction.setVisible(False)
        self.maintainAction.setVisible(False)
        self.notesAction.setVisible(False)
        self.timeAction.setVisible(False)
        if serviceItem[u'service_item'].is_capable(ItemCapabilities.CanEdit)\
            and serviceItem[u'service_item'].edit_id:
            self.editAction.setVisible(True)
        if serviceItem[u'service_item']\
            .is_capable(ItemCapabilities.CanMaintain):
            self.maintainAction.setVisible(True)
        if item.parent() is None:
            self.notesAction.setVisible(True)
        if serviceItem[u'service_item']\
            .is_capable(ItemCapabilities.HasVariableStartTime):
            self.timeAction.setVisible(True)
        self.themeMenu.menuAction().setVisible(False)
        # Set up the theme menu.
        if serviceItem[u'service_item'].is_text() and \
            self.mainwindow.renderer.theme_level == ThemeLevel.Song:
            self.themeMenu.menuAction().setVisible(True)
            # The service item does not have a theme, check the "Default".
            if serviceItem[u'service_item'].theme is None:
                themeAction = self.themeMenu.defaultAction()
            else:
                themeAction = self.themeMenu.findChild(
                    QtGui.QAction, serviceItem[u'service_item'].theme)
            if themeAction is not None:
                themeAction.setChecked(True)
        self.menu.exec_(self.serviceManagerList.mapToGlobal(point))

    def onServiceItemNoteForm(self):
        item = self.findServiceItem()[0]
        self.serviceNoteForm.textEdit.setPlainText(
            self.serviceItems[item][u'service_item'].notes)
        if self.serviceNoteForm.exec_():
            self.serviceItems[item][u'service_item'].notes = \
                unicode(self.serviceNoteForm.textEdit.toPlainText())
            self.repaintServiceList(item, -1)
            self.setModified()

    def onStartTimeForm(self):
        """
        Opens a dialog to type in service item notes.
        """
        item = self.findServiceItem()[0]
        self.startTimeForm.item = self.serviceItems[item]
        if self.startTimeForm.exec_():
            self.repaintServiceList(item, -1)

    def onServiceItemEditForm(self):
        """
        Opens a dialog to edit the service item and update the service
        display if changes are saved.
        """
        item = self.findServiceItem()[0]
        self.serviceItemEditForm.setServiceItem(
            self.serviceItems[item][u'service_item'])
        if self.serviceItemEditForm.exec_():
            self.addServiceItem(self.serviceItemEditForm.getServiceItem(),
                replace=True, expand=self.serviceItems[item][u'expanded'])

    def previewLive(self, message):
        """
        Called by the SlideController to request a preview item be made live
        and allows the next preview to be updated if relevant.
        """
        uuid, row = message.split(u':')
        for sitem in self.serviceItems:
            if sitem[u'service_item']._uuid == uuid:
                item = self.serviceManagerList.topLevelItem(sitem[u'order'] - 1)
                self.serviceManagerList.setCurrentItem(item)
                self.makeLive(int(row))
                return

    def nextItem(self):
        """
        Called by the SlideController to select the next service item.
        """
        if not self.serviceManagerList.selectedItems():
            return
        selected = self.serviceManagerList.selectedItems()[0]
        lookFor = 0
        serviceIterator = QtGui.QTreeWidgetItemIterator(self.serviceManagerList)
        while serviceIterator.value():
            if lookFor == 1 and serviceIterator.value().parent() is None:
                self.serviceManagerList.setCurrentItem(serviceIterator.value())
                self.makeLive()
                return
            if serviceIterator.value() == selected:
                lookFor = 1
            serviceIterator += 1

    def previousItem(self, message):
        """
        Called by the SlideController to select the previous service item.
        """
        if not self.serviceManagerList.selectedItems():
            return
        selected = self.serviceManagerList.selectedItems()[0]
        prevItem = None
        prevItemLastSlide = None
        serviceIterator = QtGui.QTreeWidgetItemIterator(self.serviceManagerList)
        while serviceIterator.value():
            if serviceIterator.value() == selected:
                if message == u'last slide' and prevItemLastSlide:
                    pos = prevItem.data(0, QtCore.Qt.UserRole).toInt()[0]
                    check_expanded = self.serviceItems[pos - 1][u'expanded']
                    self.serviceManagerList.setCurrentItem(prevItemLastSlide)
                    if not check_expanded:
                        self.serviceManagerList.collapseItem(prevItem)
                    self.makeLive()
                    self.serviceManagerList.setCurrentItem(prevItem)
                elif prevItem:
                    self.serviceManagerList.setCurrentItem(prevItem)
                    self.makeLive()
                return
            if serviceIterator.value().parent() is None:
                prevItem = serviceIterator.value()
            if serviceIterator.value().parent() is prevItem:
                prevItemLastSlide = serviceIterator.value()
            serviceIterator += 1

    def onSetItem(self, message):
        """
        Called by a signal to select a specific item.
        """
        self.setItem(int(message))

    def setItem(self, index):
        """
        Makes a specific item in the service live.
        """
        if index >= 0 and index < self.serviceManagerList.topLevelItemCount:
            item = self.serviceManagerList.topLevelItem(index)
            self.serviceManagerList.setCurrentItem(item)
            self.makeLive()

    def onMoveSelectionUp(self):
        """
        Moves the cursor selection up the window.
        Called by the up arrow.
        """
        item = self.serviceManagerList.currentItem()
        itemBefore = self.serviceManagerList.itemAbove(item)
        if itemBefore is None:
            return
        self.serviceManagerList.setCurrentItem(itemBefore)

    def onMoveSelectionDown(self):
        """
        Moves the cursor selection down the window.
        Called by the down arrow.
        """
        item = self.serviceManagerList.currentItem()
        itemAfter = self.serviceManagerList.itemBelow(item)
        if itemAfter is None:
            return
        self.serviceManagerList.setCurrentItem(itemAfter)

    def onCollapseAll(self):
        """
        Collapse all the service items.
        """
        for item in self.serviceItems:
            item[u'expanded'] = False
        self.serviceManagerList.collapseAll()

    def collapsed(self, item):
        """
        Record if an item is collapsed. Used when repainting the list to get the
        correct state.
        """
        pos = item.data(0, QtCore.Qt.UserRole).toInt()[0]
        self.serviceItems[pos - 1][u'expanded'] = False

    def onExpandAll(self):
        """
        Collapse all the service items.
        """
        for item in self.serviceItems:
            item[u'expanded'] = True
        self.serviceManagerList.expandAll()

    def expanded(self, item):
        """
        Record if an item is collapsed. Used when repainting the list to get the
        correct state.
        """
        pos = item.data(0, QtCore.Qt.UserRole).toInt()[0]
        self.serviceItems[pos - 1][u'expanded'] = True

    def onServiceTop(self):
        """
        Move the current ServiceItem to the top of the list.
        """
        item, child = self.findServiceItem()
        if item < len(self.serviceItems) and item != -1:
            temp = self.serviceItems[item]
            self.serviceItems.remove(self.serviceItems[item])
            self.serviceItems.insert(0, temp)
            self.repaintServiceList(0, child)
            self.setModified()

    def onServiceUp(self):
        """
        Move the current ServiceItem one position up in the list.
        """
        item, child = self.findServiceItem()
        if item > 0:
            temp = self.serviceItems[item]
            self.serviceItems.remove(self.serviceItems[item])
            self.serviceItems.insert(item - 1, temp)
            self.repaintServiceList(item - 1, child)
            self.setModified()

    def onServiceDown(self):
        """
        Move the current ServiceItem one position down in the list.
        """
        item, child = self.findServiceItem()
        if item < len(self.serviceItems) and item != -1:
            temp = self.serviceItems[item]
            self.serviceItems.remove(self.serviceItems[item])
            self.serviceItems.insert(item + 1, temp)
            self.repaintServiceList(item + 1, child)
            self.setModified()

    def onServiceEnd(self):
        """
        Move the current ServiceItem to the bottom of the list.
        """
        item, child = self.findServiceItem()
        if item < len(self.serviceItems) and item != -1:
            temp = self.serviceItems[item]
            self.serviceItems.remove(self.serviceItems[item])
            self.serviceItems.insert(len(self.serviceItems), temp)
            self.repaintServiceList(len(self.serviceItems) - 1, child)
            self.setModified()

    def onDeleteFromService(self):
        """
        Remove the current ServiceItem from the list.
        """
        item = self.findServiceItem()[0]
        if item != -1:
            self.serviceItems.remove(self.serviceItems[item])
            self.repaintServiceList(item - 1, -1)
            self.setModified()

    def repaintServiceList(self, serviceItem, serviceItemChild):
        """
        Clear the existing service list and prepaint all the items. This is
        used when moving items as the move takes place in a supporting list,
        and when regenerating all the items due to theme changes.

        ``serviceItem``
            The item which changed. (int)

        ``serviceItemChild``
            The child of the ``serviceItem``, which will be selected. (int)
        """
        # Correct order of items in array
        count = 1
        for item in self.serviceItems:
            item[u'order'] = count
            count += 1
        # Repaint the screen
        self.serviceManagerList.clear()
        for itemcount, item in enumerate(self.serviceItems):
            serviceitem = item[u'service_item']
            treewidgetitem = QtGui.QTreeWidgetItem(self.serviceManagerList)
            if serviceitem.is_valid:
                if serviceitem.notes:
                    icon = QtGui.QImage(serviceitem.icon)
                    icon = icon.scaled(80, 80, QtCore.Qt.KeepAspectRatio,
                        QtCore.Qt.SmoothTransformation)
                    overlay = QtGui.QImage(':/services/service_item_notes.png')
                    overlay = overlay.scaled(80, 80, QtCore.Qt.KeepAspectRatio,
                        QtCore.Qt.SmoothTransformation)
                    painter = QtGui.QPainter(icon)
                    painter.drawImage(0, 0, overlay)
                    painter.end()
                    treewidgetitem.setIcon(0, build_icon(icon))
                elif serviceitem.temporary_edit:
                    icon = QtGui.QImage(serviceitem.icon)
                    icon = icon.scaled(80, 80, QtCore.Qt.KeepAspectRatio,
                        QtCore.Qt.SmoothTransformation)
                    overlay = QtGui.QImage(':/general/general_export.png')
                    overlay = overlay.scaled(40, 40, QtCore.Qt.KeepAspectRatio,
                        QtCore.Qt.SmoothTransformation)
                    painter = QtGui.QPainter(icon)
                    painter.drawImage(40, 0, overlay)
                    painter.end()
                    treewidgetitem.setIcon(0, build_icon(icon))
                else:
                    treewidgetitem.setIcon(0, serviceitem.iconic_representation)
            else:
                treewidgetitem.setIcon(0,
                    build_icon(u':/general/general_delete.png'))
            treewidgetitem.setText(0, serviceitem.get_display_title())
            tips = []
            if serviceitem.temporary_edit:
                tips.append(u'<strong>%s:</strong> <em>%s</em>' %
                    (unicode(translate('OpenLP.ServiceManager', 'Edit')),
                    (unicode(translate('OpenLP.ServiceManager',
                    'Service copy only')))))
            if serviceitem.theme and serviceitem.theme != -1:
                tips.append(u'<strong>%s:</strong> <em>%s</em>' %
                    (unicode(translate('OpenLP.ServiceManager', 'Slide theme')),
                    serviceitem.theme))
            if serviceitem.notes:
                tips.append(u'<strong>%s: </strong> %s' %
                    (unicode(translate('OpenLP.ServiceManager', 'Notes')),
                    cgi.escape(unicode(serviceitem.notes))))
            if item[u'service_item'] \
                .is_capable(ItemCapabilities.HasVariableStartTime):
                tips.append(item[u'service_item'].get_media_time())
            treewidgetitem.setToolTip(0, u'<br>'.join(tips))
            treewidgetitem.setData(0, QtCore.Qt.UserRole,
                QtCore.QVariant(item[u'order']))
            treewidgetitem.setSelected(item[u'selected'])
            # Add the children to their parent treewidgetitem.
            for count, frame in enumerate(serviceitem.get_frames()):
                child = QtGui.QTreeWidgetItem(treewidgetitem)
                text = frame[u'title'].replace(u'\n', u' ')
                child.setText(0, text[:40])
                child.setData(0, QtCore.Qt.UserRole, QtCore.QVariant(count))
                if serviceItem == itemcount:
                    if item[u'expanded'] and serviceItemChild == count:
                        self.serviceManagerList.setCurrentItem(child)
                    elif serviceItemChild == -1:
                        self.serviceManagerList.setCurrentItem(treewidgetitem)
            treewidgetitem.setExpanded(item[u'expanded'])

    def validateItem(self, serviceItem):
        """
        Validates the service item and if the suffix matches an accepted
        one it allows the item to be displayed.
        """
        if serviceItem.is_command():
            type = serviceItem._raw_frames[0][u'title'].split(u'.')[-1]
            if type.lower() not in self.suffixes:
                serviceItem.is_valid = False

    def cleanUp(self):
        """
        Empties the servicePath of temporary files on system exit.
        """
        log.debug(u'Cleaning up servicePath')
        for file in os.listdir(self.servicePath):
            file_path = os.path.join(self.servicePath, file)
            delete_file(file_path)
        if os.path.exists(os.path.join(self.servicePath, u'audio')):
            shutil.rmtree(os.path.join(self.servicePath, u'audio'), True)

    def onThemeComboBoxSelected(self, currentIndex):
        """
        Set the theme for the current service.
        """
        log.debug(u'onThemeComboBoxSelected')
        self.service_theme = unicode(self.themeComboBox.currentText())
        self.mainwindow.renderer.set_service_theme(self.service_theme)
        Settings().setValue(
            self.mainwindow.serviceManagerSettingsSection +
                u'/service theme',
            QtCore.QVariant(self.service_theme))
        self.regenerateServiceItems(True)

    def themeChange(self):
        """
        The theme may have changed in the settings dialog so make
        sure the theme combo box is in the correct state.
        """
        log.debug(u'themeChange')
        visible = self.mainwindow.renderer.theme_level == ThemeLevel.Global
        self.themeLabel.setVisible(visible)
        self.themeComboBox.setVisible(visible)

    def regenerateServiceItems(self, changed=False):
        """
        Rebuild the service list as things have changed and a
        repaint is the easiest way to do this.
        """
        Receiver.send_message(u'cursor_busy')
        log.debug(u'regenerateServiceItems')
        # force reset of renderer as theme data has changed
        self.mainwindow.renderer.themedata = None
        if self.serviceItems:
            for item in self.serviceItems:
                item[u'selected'] = False
            serviceIterator = QtGui.QTreeWidgetItemIterator(
                self.serviceManagerList)
            selectedItem = None
            while serviceIterator.value():
                if serviceIterator.value().isSelected():
                    selectedItem = serviceIterator.value()
                serviceIterator += 1
            if selectedItem is not None:
                if selectedItem.parent() is None:
                    pos = selectedItem.data(0, QtCore.Qt.UserRole).toInt()[0]
                else:
                    pos = selectedItem.parent().data(0, QtCore.Qt.UserRole). \
                        toInt()[0]
                self.serviceItems[pos - 1][u'selected'] = True
            tempServiceItems = self.serviceItems
            self.serviceManagerList.clear()
            self.serviceItems = []
            self.isNew = True
            for item in tempServiceItems:
                self.addServiceItem(
                    item[u'service_item'], False, expand=item[u'expanded'],
                    repaint=False, selected=item[u'selected'])
            # Set to False as items may have changed rendering
            # does not impact the saved song so True may also be valid
            if changed:
                self.setModified()
            # Repaint it once only at the end
            self.repaintServiceList(-1, -1)
        Receiver.send_message(u'cursor_normal')

    def serviceItemUpdate(self, message):
        """
        Triggered from plugins to update service items.
        Save the values as they will be used as part of the service load
        """
        edit_id, self.load_item_uuid, temporary = message.split(u':')
        self.load_item_edit_id = int(edit_id)
        self.load_item_temporary = str_to_bool(temporary)

    def replaceServiceItem(self, newItem):
        """
        Using the service item passed replace the one with the same edit id
        if found.
        """
        for itemcount, item in enumerate(self.serviceItems):
            if item[u'service_item'].edit_id == newItem.edit_id and \
                item[u'service_item'].name == newItem.name:
                newItem.render()
                newItem.merge(item[u'service_item'])
                item[u'service_item'] = newItem
                self.repaintServiceList(itemcount + 1, 0)
                self.mainwindow.liveController.replaceServiceManagerItem(
                    newItem)
                self.setModified()

    def addServiceItem(self, item, rebuild=False, expand=None, replace=False,
        repaint=True, selected=False):
        """
        Add a Service item to the list

        ``item``
            Service Item to be added

        ``expand``
            Override the default expand settings. (Tristate)
        """
        # if not passed set to config value
        if expand is None:
            expand = self.expandTabs
        item.from_service = True
        if replace:
            sitem, child = self.findServiceItem()
            item.merge(self.serviceItems[sitem][u'service_item'])
            self.serviceItems[sitem][u'service_item'] = item
            self.repaintServiceList(sitem, child)
            self.mainwindow.liveController.replaceServiceManagerItem(item)
        else:
            item.render()
            # nothing selected for dnd
            if self.dropPosition == 0:
                if isinstance(item, list):
                    for inditem in item:
                        self.serviceItems.append({u'service_item': inditem,
                            u'order': len(self.serviceItems) + 1,
                            u'expanded': expand, u'selected': selected})
                else:
                    self.serviceItems.append({u'service_item': item,
                        u'order': len(self.serviceItems) + 1,
                        u'expanded': expand, u'selected': selected})
                if repaint:
                    self.repaintServiceList(len(self.serviceItems) - 1, -1)
            else:
                self.serviceItems.insert(self.dropPosition,
                    {u'service_item': item, u'order': self.dropPosition,
                    u'expanded': expand, u'selected': selected})
                self.repaintServiceList(self.dropPosition, -1)
            # if rebuilding list make sure live is fixed.
            if rebuild:
                self.mainwindow.liveController.replaceServiceManagerItem(item)
        self.dropPosition = 0
        self.setModified()

    def makePreview(self):
        """
        Send the current item to the Preview slide controller
        """
        Receiver.send_message(u'cursor_busy')
        item, child = self.findServiceItem()
        if self.serviceItems[item][u'service_item'].is_valid:
            self.mainwindow.previewController.addServiceManagerItem(
                self.serviceItems[item][u'service_item'], child)
        else:
            critical_error_message_box(
                translate('OpenLP.ServiceManager', 'Missing Display Handler'),
                translate('OpenLP.ServiceManager', 'Your item cannot be '
                'displayed as there is no handler to display it'))
        Receiver.send_message(u'cursor_normal')

    def getServiceItem(self):
        """
        Send the current item to the Preview slide controller
        """
        item = self.findServiceItem()[0]
        if item == -1:
            return False
        else:
            return self.serviceItems[item][u'service_item']

    def onMakeLive(self):
        """
        Send the current item to the Live slide controller but triggered
        by a tablewidget click event.
        """
        self.makeLive()

    def makeLive(self, row=-1):
        """
        Send the current item to the Live slide controller

        ``row``
            Row number to be displayed if from preview.
            -1 is passed if the value is not set
        """
        item, child = self.findServiceItem()
        # No items in service
        if item == -1:
            return
        if row != -1:
            child = row
        Receiver.send_message(u'cursor_busy')
        if self.serviceItems[item][u'service_item'].is_valid:
            self.mainwindow.liveController.addServiceManagerItem(
                self.serviceItems[item][u'service_item'], child)
            if Settings().value(
                self.mainwindow.generalSettingsSection + u'/auto preview',
                QtCore.QVariant(False)).toBool():
                item += 1
                if self.serviceItems and item < len(self.serviceItems) and \
                    self.serviceItems[item][u'service_item'].is_capable(
                    ItemCapabilities.CanPreview):
                    self.mainwindow.previewController.addServiceManagerItem(
                        self.serviceItems[item][u'service_item'], 0)
                    self.mainwindow.liveController.previewListWidget.setFocus()
        else:
            critical_error_message_box(
                translate('OpenLP.ServiceManager', 'Missing Display Handler'),
                translate('OpenLP.ServiceManager', 'Your item cannot be '
                'displayed as the plugin required to display it is missing '
                'or inactive'))
        Receiver.send_message(u'cursor_normal')

    def remoteEdit(self):
        """
        Posts a remote edit message to a plugin to allow item to be edited.
        """
        item = self.findServiceItem()[0]
        if self.serviceItems[item][u'service_item']\
            .is_capable(ItemCapabilities.CanEdit):
            Receiver.send_message(u'%s_edit' %
                self.serviceItems[item][u'service_item'].name.lower(),
                u'L:%s' % self.serviceItems[item][u'service_item'].edit_id)

    def findServiceItem(self):
        """
        Finds the first selected ServiceItem in the list and returns the
        position of the serviceitem and its selected child item. For example,
        if the third child item (in the Slidecontroller known as slide) in the
        second service item is selected this will return::

            (1, 2)
        """
        items = self.serviceManagerList.selectedItems()
        serviceItem = -1
        serviceItemChild = -1
        for item in items:
            parentitem = item.parent()
            if parentitem is None:
                serviceItem = item.data(0, QtCore.Qt.UserRole).toInt()[0]
            else:
                serviceItem = parentitem.data(0, QtCore.Qt.UserRole).toInt()[0]
                serviceItemChild = item.data(0, QtCore.Qt.UserRole).toInt()[0]
            # Adjust for zero based arrays.
            serviceItem -= 1
            # Only process the first item on the list for this method.
            break
        return serviceItem, serviceItemChild

    def dragEnterEvent(self, event):
        """
        Accept Drag events

        ``event``
            Handle of the event pint passed
        """
        event.accept()

    def dropEvent(self, event):
        """
        Receive drop event and trigger an internal event to get the
        plugins to build and push the correct service item
        The drag event payload carries the plugin name

        ``event``
            Handle of the event pint passed
        """
        link = event.mimeData()
        if link.hasUrls():
            event.setDropAction(QtCore.Qt.CopyAction)
            event.accept()
            for url in link.urls():
                filename = unicode(url.toLocalFile())
                if filename.endswith(u'.osz'):
                    self.onLoadServiceClicked(filename)
        elif link.hasText():
            plugin = unicode(link.text())
            item = self.serviceManagerList.itemAt(event.pos())
            # ServiceManager started the drag and drop
            if plugin == u'ServiceManager':
                startpos, child = self.findServiceItem()
                # If no items selected
                if startpos == -1:
                    return
                if item is None:
                    endpos = len(self.serviceItems)
                else:
                    endpos = self._getParentItemData(item) - 1
                serviceItem = self.serviceItems[startpos]
                self.serviceItems.remove(serviceItem)
                self.serviceItems.insert(endpos, serviceItem)
                self.repaintServiceList(endpos, child)
                self.setModified()
            else:
                # we are not over anything so drop
                replace = False
                if item is None:
                    self.dropPosition = len(self.serviceItems)
                else:
                    # we are over something so lets investigate
                    pos = self._getParentItemData(item) - 1
                    serviceItem = self.serviceItems[pos]
                    if (plugin == serviceItem[u'service_item'].name and
                        serviceItem[u'service_item'].is_capable(
                        ItemCapabilities.CanAppend)):
                        action = self.dndMenu.exec_(QtGui.QCursor.pos())
                        # New action required
                        if action == self.newAction:
                            self.dropPosition = self._getParentItemData(item)
                        # Append to existing action
                        if action == self.addToAction:
                            self.dropPosition = self._getParentItemData(item)
                            item.setSelected(True)
                            replace = True
                    else:
                        self.dropPosition = self._getParentItemData(item)
                Receiver.send_message(u'%s_add_service_item' % plugin, replace)

    def updateThemeList(self, theme_list):
        """
        Called from ThemeManager when the Themes have changed

        ``theme_list``
            A list of current themes to be displayed
        """
        self.themeComboBox.clear()
        self.themeMenu.clear()
        self.themeComboBox.addItem(u'')
        themeGroup = QtGui.QActionGroup(self.themeMenu)
        themeGroup.setExclusive(True)
        themeGroup.setObjectName(u'themeGroup')
        # Create a "Default" theme, which allows the user to reset the item's
        # theme to the service theme or global theme.
        defaultTheme = create_widget_action(self.themeMenu,
            text=UiStrings().Default, checked=False,
            triggers=self.onThemeChangeAction)
        self.themeMenu.setDefaultAction(defaultTheme)
        themeGroup.addAction(defaultTheme)
        self.themeMenu.addSeparator()
        for theme in theme_list:
            self.themeComboBox.addItem(theme)
            themeGroup.addAction(create_widget_action(self.themeMenu, theme,
                text=theme, checked=False, triggers=self.onThemeChangeAction))
        find_and_set_in_combo_box(self.themeComboBox, self.service_theme)
        self.mainwindow.renderer.set_service_theme(self.service_theme)
        self.regenerateServiceItems()

    def onThemeChangeAction(self):
        theme = unicode(self.sender().objectName())
        # No object name means that the "Default" theme is supposed to be used.
        if not theme:
            theme = None
        item = self.findServiceItem()[0]
        self.serviceItems[item][u'service_item'].update_theme(theme)
        self.regenerateServiceItems(True)

    def _getParentItemData(self, item):
        parentitem = item.parent()
        if parentitem is None:
            return item.data(0, QtCore.Qt.UserRole).toInt()[0]
        else:
            return parentitem.data(0, QtCore.Qt.UserRole).toInt()[0]

    def printServiceOrder(self):
        """
        Print a Service Order Sheet.
        """
        settingDialog = PrintServiceForm(self.mainwindow, self)
        settingDialog.exec_()



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

import logging
import re

from PyQt4 import QtGui

from openlp.core.lib import Receiver, translate
from openlp.core.lib.ui import UiStrings, critical_error_message_box
from editbibledialog import Ui_EditBibleDialog
from openlp.plugins.bibles.lib import BibleStrings
from openlp.plugins.bibles.lib.db import BiblesResourcesDB

log = logging.getLogger(__name__)

class EditBibleForm(QtGui.QDialog, Ui_EditBibleDialog):
    """
    Class to manage the editing of a bible
    """
    log.info(u'%s EditBibleForm loaded', __name__)

    def __init__(self, mediaitem, parent, manager):
        """
        Constructor
        """
        QtGui.QDialog.__init__(self, parent)
        self.mediaitem = mediaitem
        self.book_names = BibleStrings().BookNames
        self.setupUi(self)
        self.manager = manager

    def loadBible(self, bible):
        """
        Loads a bible.

        ``bible``
            The name of the bible.
        """
        log.debug(u'Load Bible')
        self.bible = bible
        self.versionNameEdit.setText(
            self.manager.get_meta_data(self.bible, u'name').value)
        self.copyrightEdit.setText(
            self.manager.get_meta_data(self.bible, u'copyright').value)
        self.permissionsEdit.setText(
            self.manager.get_meta_data(self.bible, u'permissions').value)
        book_name_language = self.manager.get_meta_data(self.bible,
            u'book_name_language')
        if book_name_language and book_name_language.value != u'None':
            self.languageSelectionComboBox.setCurrentIndex(
                int(book_name_language.value) + 1)
        self.books = {}
        self.webbible = self.manager.get_meta_data(self.bible,
            u'download_source')
        if self.webbible:
            self.bookNameNotice.setText(translate('BiblesPlugin.EditBibleForm',
                'This is a Web Download Bible.\nIt is not possible to '
                'customize the Book Names.'))
            self.scrollArea.hide()
        else:
            self.bookNameNotice.setText(translate('BiblesPlugin.EditBibleForm',
                'To use the customized book names, "Bible language" must be '
                'selected on the Meta Data tab or, if "Global settings" is '
                'selected, on the Bible page in Configure OpenLP.'))
            for book in BiblesResourcesDB.get_books():
                self.books[book[u'abbreviation']] = self.manager.get_book_by_id(
                    self.bible, book[u'id'])
                if self.books[book[u'abbreviation']] and not self.webbible:
                    self.bookNameEdit[book[u'abbreviation']].setText(
                        self.books[book[u'abbreviation']].name)
                else:
                    # It is nessecary to remove the Widget otherwise there still
                    # exists the vertical spacing in QFormLayout
                    self.bookNameWidgetLayout.removeWidget(
                        self.bookNameLabel[book[u'abbreviation']])
                    self.bookNameLabel[book[u'abbreviation']].hide()
                    self.bookNameWidgetLayout.removeWidget(
                        self.bookNameEdit[book[u'abbreviation']])
                    self.bookNameEdit[book[u'abbreviation']].hide()

    def reject(self):
        """
        Exit Dialog and do not save
        """
        log.debug(u'BibleEditForm.reject')
        self.bible = None
        QtGui.QDialog.reject(self)

    def accept(self):
        """
        Exit Dialog and save data
        """
        log.debug(u'BibleEditForm.accept')
        version = unicode(self.versionNameEdit.text())
        copyright = unicode(self.copyrightEdit.text())
        permissions = unicode(self.permissionsEdit.text())
        book_name_language = self.languageSelectionComboBox.currentIndex() - 1
        if book_name_language == -1:
            book_name_language = None
        if not self.validateMeta(version, copyright):
            return
        if not self.webbible:
            custom_names = {}
            for abbr, book in self.books.iteritems():
                if book:
                    custom_names[abbr] = unicode(self.bookNameEdit[abbr].text())
                    if book.name != custom_names[abbr]:
                        if not self.validateBook(custom_names[abbr], abbr):
                            return
        Receiver.send_message(u'openlp_process_events')
        Receiver.send_message(u'cursor_busy')
        self.manager.save_meta_data(self.bible, version, copyright, permissions,
            book_name_language)
        if not self.webbible:
            for abbr, book in self.books.iteritems():
                if book:
                    if book.name != custom_names[abbr]:
                        book.name = custom_names[abbr]
                        self.manager.update_book(self.bible, book)
        self.bible = None
        Receiver.send_message(u'cursor_normal')
        QtGui.QDialog.accept(self)

    def validateMeta(self, name, copyright):
        """
        Validate the Meta before saving.
        """
        if not name:
            self.versionNameEdit.setFocus()
            critical_error_message_box(UiStrings().EmptyField,
                translate('BiblesPlugin.BibleEditForm',
                'You need to specify a version name for your Bible.'))
            return False
        elif not copyright:
            self.copyrightEdit.setFocus()
            critical_error_message_box(UiStrings().EmptyField,
                translate('BiblesPlugin.BibleEditForm',
                'You need to set a copyright for your Bible. '
                'Bibles in the Public Domain need to be marked as such.'))
            return False
        elif self.manager.exists(name) and \
            self.manager.get_meta_data(self.bible, u'name').value != \
            name:
            self.versionNameEdit.setFocus()
            critical_error_message_box(
                translate('BiblesPlugin.BibleEditForm', 'Bible Exists'),
                translate('BiblesPlugin.BibleEditForm',
                'This Bible already exists. Please import '
                'a different Bible or first delete the existing one.'))
            return False
        return True

    def validateBook(self, new_book_name, abbreviation):
        """
        Validate a book.
        """
        book_regex = re.compile(u'[\d]*[^\d]+$')
        if not new_book_name:
            self.bookNameEdit[abbreviation].setFocus()
            critical_error_message_box(UiStrings().EmptyField,
                unicode(translate('BiblesPlugin.BibleEditForm',
                'You need to specify a book name for "%s".')) %
                self.book_names[abbreviation])
            return False
        elif not book_regex.match(new_book_name):
            self.bookNameEdit[abbreviation].setFocus()
            critical_error_message_box(UiStrings().EmptyField,
                unicode(translate('BiblesPlugin.BibleEditForm',
                'The book name "%s" is not correct.\nNumbers can only be used '
                'at the beginning and must\nbe followed by one or more '
                'non-numeric characters.')) % new_book_name)
            return False
        for abbr, book in self.books.iteritems():
            if book:
                if abbr == abbreviation:
                    continue
                if unicode(self.bookNameEdit[abbr].text()) == new_book_name:
                    self.bookNameEdit[abbreviation].setFocus()
                    critical_error_message_box(
                        translate('BiblesPlugin.BibleEditForm',
                        'Duplicate Book Name'),
                        unicode(translate('BiblesPlugin.BibleEditForm',
                        'The Book Name "%s" has been entered more than once.'))
                        % new_book_name)
                    return False
        return True



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

import logging

from PyQt4 import QtCore, QtGui

from openlp.core.lib import Receiver, translate
from openlp.core.lib.ui import critical_error_message_box, \
    find_and_set_in_combo_box
from openlp.plugins.custom.lib import CustomXMLBuilder, CustomXMLParser
from openlp.plugins.custom.lib.db import CustomSlide
from editcustomdialog import Ui_CustomEditDialog
from editcustomslideform import EditCustomSlideForm

log = logging.getLogger(__name__)

class EditCustomForm(QtGui.QDialog, Ui_CustomEditDialog):
    """
    Class documentation goes here.
    """
    log.info(u'Custom Editor loaded')
    def __init__(self, mediaitem, parent, manager):
        """
        Constructor
        """
        QtGui.QDialog.__init__(self, parent)
        self.manager = manager
        self.mediaitem = mediaitem
        self.setupUi(self)
        # Create other objects and forms.
        self.editSlideForm = EditCustomSlideForm(self)
        # Connecting signals and slots
        QtCore.QObject.connect(self.previewButton,
            QtCore.SIGNAL(u'clicked()'), self.onPreviewButtonClicked)
        QtCore.QObject.connect(self.addButton,
            QtCore.SIGNAL(u'clicked()'), self.onAddButtonClicked)
        QtCore.QObject.connect(self.editButton,
            QtCore.SIGNAL(u'clicked()'), self.onEditButtonClicked)
        QtCore.QObject.connect(self.editAllButton,
            QtCore.SIGNAL(u'clicked()'), self.onEditAllButtonClicked)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'theme_update_list'), self.loadThemes)
        QtCore.QObject.connect(self.slideListView,
            QtCore.SIGNAL(u'currentRowChanged(int)'), self.onCurrentRowChanged)
        QtCore.QObject.connect(self.slideListView,
            QtCore.SIGNAL(u'doubleClicked(QModelIndex)'),
            self.onEditButtonClicked)

    def loadThemes(self, themelist):
        self.themeComboBox.clear()
        self.themeComboBox.addItem(u'')
        for themename in themelist:
            self.themeComboBox.addItem(themename)

    def loadCustom(self, id, preview=False):
        """
        Called when editing or creating a new custom.

        ``id``
            The cutom's id. If zero, then a new custom is created.

        ``preview``
            States whether the custom is edited while being previewed in the
            preview panel.
        """
        self.slideListView.clear()
        if id == 0:
            self.customSlide = CustomSlide()
            self.titleEdit.setText(u'')
            self.creditEdit.setText(u'')
            self.themeComboBox.setCurrentIndex(0)
        else:
            self.customSlide = self.manager.get_object(CustomSlide, id)
            self.titleEdit.setText(self.customSlide.title)
            self.creditEdit.setText(self.customSlide.credits)
            customXML = CustomXMLParser(self.customSlide.text)
            slideList = customXML.get_verses()
            for slide in slideList:
                self.slideListView.addItem(slide[1])
            theme = self.customSlide.theme_name
            find_and_set_in_combo_box(self.themeComboBox, theme)
        self.titleEdit.setFocus(QtCore.Qt.OtherFocusReason)
        # If not preview hide the preview button.
        self.previewButton.setVisible(preview)

    def reject(self):
        Receiver.send_message(u'custom_edit_clear')
        QtGui.QDialog.reject(self)

    def accept(self):
        log.debug(u'accept')
        if self.saveCustom():
            QtGui.QDialog.accept(self)

    def saveCustom(self):
        """
        Saves the custom.
        """
        if not self._validate():
            return False
        sxml = CustomXMLBuilder()
        sxml.new_document()
        sxml.add_lyrics_to_song()
        count = 1
        for i in range(self.slideListView.count()):
            sxml.add_verse_to_lyrics(u'custom', unicode(count),
                unicode(self.slideListView.item(i).text()))
            count += 1
        self.customSlide.title = unicode(self.titleEdit.text())
        self.customSlide.text = unicode(sxml.extract_xml(), u'utf-8')
        self.customSlide.credits = unicode(self.creditEdit.text())
        self.customSlide.theme_name = unicode(self.themeComboBox.currentText())
        success = self.manager.save_object(self.customSlide)
        self.mediaitem.autoSelectId = self.customSlide.id
        return success

    def onUpButtonClicked(self):
        selectedRow = self.slideListView.currentRow()
        if selectedRow != 0:
            qw = self.slideListView.takeItem(selectedRow)
            self.slideListView.insertItem(selectedRow - 1, qw)
            self.slideListView.setCurrentRow(selectedRow - 1)

    def onDownButtonClicked(self):
        selectedRow = self.slideListView.currentRow()
        # zero base arrays
        if selectedRow != self.slideListView.count() - 1:
            qw = self.slideListView.takeItem(selectedRow)
            self.slideListView.insertItem(selectedRow + 1, qw)
            self.slideListView.setCurrentRow(selectedRow + 1)

    def onAddButtonClicked(self):
        self.editSlideForm.setText(u'')
        if self.editSlideForm.exec_():
            for slide in self.editSlideForm.getText():
                self.slideListView.addItem(slide)

    def onEditButtonClicked(self):
        self.editSlideForm.setText(self.slideListView.currentItem().text())
        if self.editSlideForm.exec_():
            self.updateSlideList(self.editSlideForm.getText())

    def onEditAllButtonClicked(self):
        """
        Edits all slides.
        """
        slide_list = u''
        for row in range(self.slideListView.count()):
            item = self.slideListView.item(row)
            slide_list += item.text()
            if row != self.slideListView.count() - 1:
                slide_list += u'\n[===]\n'
        self.editSlideForm.setText(slide_list)
        if self.editSlideForm.exec_():
            self.updateSlideList(self.editSlideForm.getText(), True)

    def onPreviewButtonClicked(self):
        """
        Save the custom item and preview it.
        """
        log.debug(u'onPreview')
        if self.saveCustom():
            Receiver.send_message(u'custom_preview')

    def updateSlideList(self, slides, edit_all=False):
        """
        Updates the slide list after editing slides.

        ``slides``
            A list of all slides which have been edited.

        ``edit_all``
            Indicates if all slides or only one slide has been edited.
        """
        if edit_all:
            self.slideListView.clear()
            for slide in slides:
                self.slideListView.addItem(slide)
        else:
            old_slides = []
            old_row = self.slideListView.currentRow()
            # Create a list with all (old/unedited) slides.
            old_slides = [self.slideListView.item(row).text() for row in \
                range(self.slideListView.count())]
            self.slideListView.clear()
            old_slides.pop(old_row)
            # Insert all slides to make the old_slides list complete.
            for slide in slides:
                old_slides.insert(old_row, slide)
            for slide in old_slides:
                self.slideListView.addItem(slide)
        self.slideListView.repaint()

    def onDeleteButtonClicked(self):
        """
        Removes the current row from the list.
        """
        self.slideListView.takeItem(self.slideListView.currentRow())
        self.onCurrentRowChanged(self.slideListView.currentRow())

    def onCurrentRowChanged(self, row):
        """
        Called when the *slideListView*'s current row has been changed. This
        enables or disables buttons which require an slide to act on.

        ``row``
            The row (int). If there is no current row, the value is -1.
        """
        if row == -1:
            self.deleteButton.setEnabled(False)
            self.editButton.setEnabled(False)
            self.upButton.setEnabled(False)
            self.downButton.setEnabled(False)
        else:
            self.deleteButton.setEnabled(True)
            self.editButton.setEnabled(True)
            # Decide if the up/down buttons should be enabled or not.
            if self.slideListView.count() - 1 == row:
                self.downButton.setEnabled(False)
            else:
                self.downButton.setEnabled(True)
            if row == 0:
                self.upButton.setEnabled(False)
            else:
                self.upButton.setEnabled(True)

    def _validate(self):
        """
        Checks whether a custom is valid or not.
        """
        # We must have a title.
        if not self.titleEdit.displayText():
            self.titleEdit.setFocus()
            critical_error_message_box(
                message=translate('CustomPlugin.EditCustomForm',
                'You need to type in a title.'))
            return False
        # We must have at least one slide.
        if self.slideListView.count() == 0:
            critical_error_message_box(
                message=translate('CustomPlugin.EditCustomForm',
                'You need to add at least one slide'))
            return False
        return True



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

import logging

from PyQt4 import QtCore, QtGui

from openlp.core.lib import Plugin, StringContent, build_icon, translate
from openlp.core.lib.ui import create_action, UiStrings
from openlp.core.lib.settings import Settings
from openlp.core.utils.actions import ActionList
from openlp.plugins.bibles.lib import BibleManager, BiblesTab, BibleMediaItem
from openlp.plugins.bibles.forms import BibleUpgradeForm

log = logging.getLogger(__name__)

class BiblePlugin(Plugin):
    log.info(u'Bible Plugin loaded')

    def __init__(self, plugin_helpers):
        Plugin.__init__(self, u'bibles', plugin_helpers,
            BibleMediaItem, BiblesTab)
        self.weight = -9
        self.iconPath = u':/plugins/plugin_bibles.png'
        self.icon = build_icon(self.iconPath)
        self.manager = None

    def initialise(self):
        log.info(u'bibles Initialising')
        if self.manager is None:
            self.manager = BibleManager(self)
        Plugin.initialise(self)
        self.importBibleItem.setVisible(True)
        action_list = ActionList.get_instance()
        action_list.add_action(self.importBibleItem,
            unicode(UiStrings().Import))
        # Do not add the action to the list yet.
        #action_list.add_action(self.exportBibleItem,
        #    unicode(UiStrings().Export))
        # Set to invisible until we can export bibles
        self.exportBibleItem.setVisible(False)
        self.toolsUpgradeItem.setVisible(bool(self.manager.old_bible_databases))

    def finalise(self):
        """
        Tidy up on exit
        """
        log.info(u'Plugin Finalise')
        self.manager.finalise()
        Plugin.finalise(self)
        action_list = ActionList.get_instance()
        action_list.remove_action(self.importBibleItem,
            unicode(UiStrings().Import))
        self.importBibleItem.setVisible(False)
        #action_list.remove_action(self.exportBibleItem, UiStrings().Export)
        self.exportBibleItem.setVisible(False)

    def appStartup(self):
        """
        Perform tasks on application startup
        """
        if self.manager.old_bible_databases:
            if QtGui.QMessageBox.information(self.formParent,
                translate('OpenLP', 'Information'), translate('OpenLP',
                'Bible format has changed.\nYou have to upgrade your '
                'existing Bibles.\nShould OpenLP upgrade now?'),
                QtGui.QMessageBox.StandardButtons(QtGui.QMessageBox.Yes |
                QtGui.QMessageBox.No)) == QtGui.QMessageBox.Yes:
                self.onToolsUpgradeItemTriggered()
        settings = Settings()
        settings.beginGroup(self.settingsSection)
        if settings.contains(u'bookname language'):
            settings.setValue(u'book name language', settings.value(
                u'bookname language', QtCore.QVariant(0)).toInt()[0])
            settings.remove(u'bookname language')
        settings.endGroup()

    def addImportMenuItem(self, import_menu):
        self.importBibleItem = create_action(import_menu, u'importBibleItem',
            text=translate('BiblesPlugin', '&Bible'), visible=False,
            triggers=self.onBibleImportClick)
        import_menu.addAction(self.importBibleItem)

    def addExportMenuItem(self, export_menu):
        self.exportBibleItem = create_action(export_menu, u'exportBibleItem',
            text=translate('BiblesPlugin', '&Bible'),
            visible=False)
        export_menu.addAction(self.exportBibleItem)

    def addToolsMenuItem(self, tools_menu):
        """
        Give the bible plugin the opportunity to add items to the
        **Tools** menu.

        ``tools_menu``
            The actual **Tools** menu item, so that your actions can
            use it as their parent.
        """
        log.debug(u'add tools menu')
        self.toolsUpgradeItem = create_action(tools_menu, u'toolsUpgradeItem',
            text=translate('BiblesPlugin', '&Upgrade older Bibles'),
            statustip=translate('BiblesPlugin',
            'Upgrade the Bible databases to the latest format.'),
            visible=False, triggers=self.onToolsUpgradeItemTriggered)
        tools_menu.addAction(self.toolsUpgradeItem)

    def onToolsUpgradeItemTriggered(self):
        """
        Upgrade older bible databases.
        """
        if not hasattr(self, u'upgrade_wizard'):
            self.upgrade_wizard = BibleUpgradeForm(self.formParent,
                self.manager, self)
        # If the import was not cancelled then reload.
        if self.upgrade_wizard.exec_():
            self.mediaItem.reloadBibles()

    def onBibleImportClick(self):
        if self.mediaItem:
            self.mediaItem.onImportClick()

    def about(self):
        about_text = translate('BiblesPlugin', '<strong>Bible Plugin</strong>'
            '<br />The Bible plugin provides the ability to display Bible '
            'verses from different sources during the service.')
        return about_text

    def usesTheme(self, theme):
        """
        Called to find out if the bible plugin is currently using a theme.
        Returns ``True`` if the theme is being used, otherwise returns
        ``False``.
        """
        return unicode(self.settingsTab.bible_theme) == theme

    def renameTheme(self, oldTheme, newTheme):
        """
        Rename the theme the bible plugin is using making the plugin use the
        new name.

        ``oldTheme``
            The name of the theme the plugin should stop using. Unused for
            this particular plugin.

        ``newTheme``
            The new name the plugin should now use.
        """
        self.settingsTab.bible_theme = newTheme
        self.settingsTab.save()

    def setPluginTextStrings(self):
        """
        Called to define all translatable texts of the plugin
        """
        ## Name PluginList ##
        self.textStrings[StringContent.Name] = {
            u'singular': translate('BiblesPlugin', 'Bible', 'name singular'),
            u'plural': translate('BiblesPlugin', 'Bibles', 'name plural')
        }
        ## Name for MediaDockManager, SettingsManager ##
        self.textStrings[StringContent.VisibleName] = {
            u'title': translate('BiblesPlugin', 'Bibles', 'container title')
        }
        # Middle Header Bar
        tooltips = {
            u'load': u'',
            u'import': translate('BiblesPlugin', 'Import a Bible.'),
            u'new': translate('BiblesPlugin', 'Add a new Bible.'),
            u'edit': translate('BiblesPlugin', 'Edit the selected Bible.'),
            u'delete': translate('BiblesPlugin', 'Delete the selected Bible.'),
            u'preview': translate('BiblesPlugin',
                'Preview the selected Bible.'),
            u'live': translate('BiblesPlugin', 'Send the selected Bible live.'),
            u'service': translate('BiblesPlugin',
                'Add the selected Bible to the service.')
        }
        self.setPluginUiTextStrings(tooltips)



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################
import logging
import re
import os
import platform

import sqlalchemy
import BeautifulSoup
from lxml import etree
from PyQt4 import Qt, QtCore, QtGui, QtWebKit

try:
    from PyQt4.phonon import Phonon
    PHONON_VERSION = Phonon.phononVersion()
except ImportError:
    PHONON_VERSION = u'-'
try:
    import migrate
    MIGRATE_VERSION = getattr(migrate, u'__version__', u'< 0.7')
except ImportError:
    MIGRATE_VERSION = u'-'
try:
    import chardet
    CHARDET_VERSION = chardet.__version__
except ImportError:
    CHARDET_VERSION = u'-'
try:
    import enchant
    ENCHANT_VERSION = enchant.__version__
except ImportError:
    ENCHANT_VERSION = u'-'
try:
    import sqlite
    SQLITE_VERSION = sqlite.version
except ImportError:
    SQLITE_VERSION = u'-'
try:
    import mako
    MAKO_VERSION = mako.__version__
except ImportError:
    MAKO_VERSION = u'-'
try:
    import uno
    arg = uno.createUnoStruct(u'com.sun.star.beans.PropertyValue')
    arg.Name = u'nodepath'
    arg.Value = u'/org.openoffice.Setup/Product'
    context = uno.getComponentContext()
    provider = context.ServiceManager.createInstance(
        u'com.sun.star.configuration.ConfigurationProvider')
    node = provider.createInstanceWithArguments(
        u'com.sun.star.configuration.ConfigurationAccess', (arg,))
    UNO_VERSION = node.getByName(u'ooSetupVersion')
except ImportError:
    UNO_VERSION = u'-'
except:
    UNO_VERSION = u'- (Possible non-standard UNO installation)'
try:
    WEBKIT_VERSION = QtWebKit.qWebKitVersion()
except AttributeError:
    WEBKIT_VERSION = u'-'


from openlp.core.lib import translate, SettingsManager
from openlp.core.lib.ui import UiStrings
from openlp.core.utils import get_application_version

from exceptiondialog import Ui_ExceptionDialog

log = logging.getLogger(__name__)

class ExceptionForm(QtGui.QDialog, Ui_ExceptionDialog):
    """
    The exception dialog
    """
    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.settingsSection = u'crashreport'

    def exec_(self):
        self.descriptionTextEdit.setPlainText(u'')
        self.onDescriptionUpdated()
        self.fileAttachment = None
        return QtGui.QDialog.exec_(self)

    def _createReport(self):
        openlp_version = get_application_version()
        description = unicode(self.descriptionTextEdit.toPlainText())
        traceback = unicode(self.exceptionTextEdit.toPlainText())
        system = unicode(translate('OpenLP.ExceptionForm',
            'Platform: %s\n')) % platform.platform()
        libraries = u'Python: %s\n' % platform.python_version() + \
            u'Qt4: %s\n' % Qt.qVersion() + \
            u'Phonon: %s\n' % PHONON_VERSION + \
            u'PyQt4: %s\n' % Qt.PYQT_VERSION_STR + \
            u'QtWebkit: %s\n' % WEBKIT_VERSION + \
            u'SQLAlchemy: %s\n' % sqlalchemy.__version__ + \
            u'SQLAlchemy Migrate: %s\n' % MIGRATE_VERSION + \
            u'BeautifulSoup: %s\n' % BeautifulSoup.__version__ + \
            u'lxml: %s\n' % etree.__version__ + \
            u'Chardet: %s\n' % CHARDET_VERSION + \
            u'PyEnchant: %s\n' % ENCHANT_VERSION + \
            u'PySQLite: %s\n' % SQLITE_VERSION + \
            u'Mako: %s\n' % MAKO_VERSION + \
            u'pyUNO bridge: %s\n' % UNO_VERSION
        if platform.system() == u'Linux':
            if os.environ.get(u'KDE_FULL_SESSION') == u'true':
                system = system + u'Desktop: KDE SC\n'
            elif os.environ.get(u'GNOME_DESKTOP_SESSION_ID'):
                system = system + u'Desktop: GNOME\n'
        return (openlp_version, description, traceback, system, libraries)

    def onSaveReportButtonClicked(self):
        """
        Saving exception log and system informations to a file.
        """
        report_text = unicode(translate('OpenLP.ExceptionForm',
            '**OpenLP Bug Report**\n'
            'Version: %s\n\n'
            '--- Details of the Exception. ---\n\n%s\n\n '
            '--- Exception Traceback ---\n%s\n'
            '--- System information ---\n%s\n'
            '--- Library Versions ---\n%s\n'))
        filename = QtGui.QFileDialog.getSaveFileName(self,
            translate('OpenLP.ExceptionForm', 'Save Crash Report'),
            SettingsManager.get_last_dir(self.settingsSection),
            translate('OpenLP.ExceptionForm',
            'Text files (*.txt *.log *.text)'))
        if filename:
            filename = unicode(filename).replace(u'/', os.path.sep)
            SettingsManager.set_last_dir(self.settingsSection, os.path.dirname(
                filename))
            report_text = report_text % self._createReport()
            try:
                report_file = open(filename, u'w')
                try:
                    report_file.write(report_text)
                except UnicodeError:
                    report_file.close()
                    report_file = open(filename, u'wb')
                    report_file.write(report_text.encode(u'utf-8'))
                finally:
                    report_file.close()
            except IOError:
                log.exception(u'Failed to write crash report')
            finally:
                report_file.close()

    def onSendReportButtonClicked(self):
        """
        Opening systems default email client and inserting exception log and
        system informations.
        """
        body = unicode(translate('OpenLP.ExceptionForm',
            '*OpenLP Bug Report*\n'
            'Version: %s\n\n'
            '--- Details of the Exception. ---\n\n%s\n\n '
            '--- Exception Traceback ---\n%s\n'
            '--- System information ---\n%s\n'
            '--- Library Versions ---\n%s\n',
            'Please add the information that bug reports are favoured written '
            'in English.'))
        content = self._createReport()
        source = u''
        exception = u''
        for line in content[2].split(u'\n'):
            if re.search(r'[/\\]openlp[/\\]', line):
                source = re.sub(r'.*[/\\]openlp[/\\](.*)".*', r'\1', line)
            if u':' in line:
                exception = line.split(u'\n')[-1].split(u':')[0]
        subject = u'Bug report: %s in %s' % (exception, source)
        mailto_url = QtCore.QUrl(u'mailto:bugs@openlp.org')
        mailto_url.addQueryItem(u'subject', subject)
        mailto_url.addQueryItem(u'body', body % content)
        if self.fileAttachment:
            mailto_url.addQueryItem(u'attach', self.fileAttachment)
        QtGui.QDesktopServices.openUrl(mailto_url)

    def onDescriptionUpdated(self):
        count = int(20 - len(self.descriptionTextEdit.toPlainText()))
        if count < 0:
            count = 0
            self.__buttonState(True)
        else:
            self.__buttonState(False)
        self.descriptionWordCount.setText(
            unicode(translate('OpenLP.ExceptionDialog',
            'Description characters to enter : %s')) % count)

    def onAttachFileButtonClicked(self):
        files = QtGui.QFileDialog.getOpenFileName(
            self,translate('ImagePlugin.ExceptionDialog',
            'Select Attachment'),
            SettingsManager.get_last_dir(u'exceptions'),
            u'%s (*.*)' % UiStrings().AllFiles)
        log.info(u'New files(s) %s', unicode(files))
        if files:
            self.fileAttachment = unicode(files)

    def __buttonState(self, state):
        self.saveReportButton.setEnabled(state)
        self.sendReportButton.setEnabled(state)




# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

from openlp.plugins.songs.lib.opensongimport import OpenSongImport
from openlp.plugins.songs.lib.db import init_schema
from openlp.core.lib.db import Manager
from glob import glob
from zipfile import ZipFile
import os
from traceback import print_exc
import sys
import codecs

import logging
LOG_FILENAME = 'import.log'
logging.basicConfig(filename=LOG_FILENAME,level=logging.INFO)

from test_opensongimport import wizard_stub, progbar_stub

# Useful test function for importing a variety of different files
# Uncomment below depending on what problem trying to make occur!

def opensong_import_lots():
    ziploc = u'/home/mjt/openlp/OpenSong_Data/'
    files = []
    files = [os.path.join(ziploc, u'RaoulSongs', u'Songs', u'Jesus Freak')]
    # files.extend(glob(ziploc+u'Songs.zip'))
    # files.extend(glob(ziploc+u'RaoulSongs.zip'))
    # files.extend(glob(ziploc+u'SOF.zip'))
    # files.extend(glob(ziploc+u'spanish_songs_for_opensong.zip'))
    # files.extend(glob(ziploc+u'opensong_*.zip'))
    errfile = codecs.open(u'import_lots_errors.txt', u'w', u'utf8')
    manager = Manager(u'songs', init_schema)
    o = OpenSongImport(manager, filenames=files)
    o.import_wizard=wizard_stub()
    o.do_import()

if __name__ == "__main__":
    opensong_import_lots()



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

from PyQt4 import QtCore, QtGui

from serviceitemeditdialog import Ui_ServiceItemEditDialog

class ServiceItemEditForm(QtGui.QDialog, Ui_ServiceItemEditDialog):
    """
    This is the form that is used to edit the verses of the song.
    """
    def __init__(self, parent=None):
        """
        Constructor
        """
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.itemList = []
        QtCore.QObject.connect(self.listWidget,
            QtCore.SIGNAL(u'currentRowChanged(int)'), self.onCurrentRowChanged)

    def setServiceItem(self, item):
        self.item = item
        self.itemList = []
        if self.item.is_image():
            self.data = True
            for frame in self.item._raw_frames:
                self.itemList.append(frame)
        self.loadData()
        self.listWidget.setCurrentItem(self.listWidget.currentItem())

    def getServiceItem(self):
        if self.data:
            self.item._raw_frames = []
            if self.item.is_image():
                for item in self.itemList:
                    self.item.add_from_image(item[u'path'], item[u'title'])
            self.item.render()
        return self.item

    def loadData(self):
        """
        Loads the image list.
        """
        self.listWidget.clear()
        for frame in self.itemList:
            item_name = QtGui.QListWidgetItem(frame[u'title'])
            self.listWidget.addItem(item_name)

    def onDeleteButtonClicked(self):
        """
        Delete the current row.
        """
        item = self.listWidget.currentItem()
        if not item:
            return
        row = self.listWidget.row(item)
        self.itemList.pop(row)
        self.loadData()
        if row == self.listWidget.count():
            self.listWidget.setCurrentRow(row - 1)
        else:
            self.listWidget.setCurrentRow(row)

    def onUpButtonClicked(self):
        """
        Move the current row up in the list.
        """
        self.__moveItem(u'up')

    def onDownButtonClicked(self):
        """
        Move the current row down in the list
        """
        self.__moveItem(u'down')

    def __moveItem(self, direction=u''):
        """
        Move the current item.
        """
        if not direction:
            return
        item = self.listWidget.currentItem()
        if not item:
            return
        row = self.listWidget.row(item)
        temp = self.itemList[row]
        self.itemList.pop(row)
        if direction == u'up':
            row -= 1
        else:
            row += 1
        self.itemList.insert(row, temp)
        self.loadData()
        self.listWidget.setCurrentRow(row)

    def onCurrentRowChanged(self, row):
        """
        Called when the currentRow has changed.

        ``row``
            The row number (int).
        """
        # Disable all buttons, as no row is selected or only one image is left.
        if row == -1 or self.listWidget.count() == 1:
            self.downButton.setEnabled(False)
            self.upButton.setEnabled(False)
            self.deleteButton.setEnabled(False)
        else:
            # Check if we are at the end of the list.
            if self.listWidget.count() == row + 1:
                self.downButton.setEnabled(False)
            else:
                self.downButton.setEnabled(True)
            # Check if we are at the beginning of the list.
            if row == 0:
                self.upButton.setEnabled(False)
            else:
                self.upButton.setEnabled(True)
            self.deleteButton.setEnabled(True)



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

from PyQt4 import QtCore, QtGui

from openlp.core.lib import build_icon, translate, SpellTextEdit
from openlp.core.lib.ui import UiStrings


class ZoomSize(object):
    """
    Type enumeration for Combo Box sizes
    """
    Page = 0
    Width = 1
    OneHundred = 2
    SeventyFive = 3
    Fifty = 4
    TwentyFive = 5


class Ui_PrintServiceDialog(object):
    def setupUi(self, printServiceDialog):
        printServiceDialog.setObjectName(u'printServiceDialog')
        printServiceDialog.resize(664, 594)
        printServiceDialog.setWindowIcon(build_icon(u':/icon/openlp-logo.svg'))
        self.mainLayout = QtGui.QVBoxLayout(printServiceDialog)
        self.mainLayout.setSpacing(0)
        self.mainLayout.setMargin(0)
        self.mainLayout.setObjectName(u'mainLayout')
        self.toolbar = QtGui.QToolBar(printServiceDialog)
        self.toolbar.setIconSize(QtCore.QSize(22, 22))
        self.toolbar.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
        self.printButton = self.toolbar.addAction(
            build_icon(u':/general/general_print.png'),
            translate('OpenLP.PrintServiceForm', 'Print'))
        self.optionsButton = QtGui.QToolButton(self.toolbar)
        self.optionsButton.setToolButtonStyle(
            QtCore.Qt.ToolButtonTextBesideIcon)
        self.optionsButton.setIcon(build_icon(u':/system/system_configure.png'))
        self.optionsButton.setCheckable(True)
        self.toolbar.addWidget(self.optionsButton)
        self.toolbar.addSeparator()
        self.plainCopy = self.toolbar.addAction(
            build_icon(u':/system/system_edit_copy.png'),
            translate('OpenLP.PrintServiceForm', 'Copy'))
        self.htmlCopy = self.toolbar.addAction(
            build_icon(u':/system/system_edit_copy.png'),
            translate('OpenLP.PrintServiceForm', 'Copy as HTML'))
        self.toolbar.addSeparator()
        self.zoomInButton = QtGui.QToolButton(self.toolbar)
        self.zoomInButton.setIcon(build_icon(u':/general/general_zoom_in.png'))
        self.zoomInButton.setObjectName(u'zoomInButton')
        self.zoomInButton.setIconSize(QtCore.QSize(22, 22))
        self.toolbar.addWidget(self.zoomInButton)
        self.zoomOutButton = QtGui.QToolButton(self.toolbar)
        self.zoomOutButton.setIcon(
            build_icon(u':/general/general_zoom_out.png'))
        self.zoomOutButton.setObjectName(u'zoomOutButton')
        self.zoomOutButton.setIconSize(QtCore.QSize(22, 22))
        self.toolbar.addWidget(self.zoomOutButton)
        self.zoomOriginalButton = QtGui.QToolButton(self.toolbar)
        self.zoomOriginalButton.setIcon(
            build_icon(u':/general/general_zoom_original.png'))
        self.zoomOriginalButton.setObjectName(u'zoomOriginalButton')
        self.zoomOriginalButton.setIconSize(QtCore.QSize(22, 22))
        self.toolbar.addWidget(self.zoomOriginalButton)
        self.zoomComboBox = QtGui.QComboBox(printServiceDialog)
        self.zoomComboBox.setObjectName(u'zoomComboBox')
        self.toolbar.addWidget(self.zoomComboBox)
        self.mainLayout.addWidget(self.toolbar)
        self.previewWidget = QtGui.QPrintPreviewWidget(printServiceDialog)
        self.mainLayout.addWidget(self.previewWidget)
        self.optionsWidget = QtGui.QWidget(printServiceDialog)
        self.optionsWidget.hide()
        self.optionsWidget.resize(400, 300)
        self.optionsWidget.setAutoFillBackground(True)
        self.optionsLayout = QtGui.QVBoxLayout(self.optionsWidget)
        self.optionsLayout.setContentsMargins(8, 8, 8, 8)
        self.titleLabel = QtGui.QLabel(self.optionsWidget)
        self.titleLabel.setObjectName(u'titleLabel')
        self.optionsLayout.addWidget(self.titleLabel)
        self.titleLineEdit = QtGui.QLineEdit(self.optionsWidget)
        self.titleLineEdit.setObjectName(u'titleLineEdit')
        self.optionsLayout.addWidget(self.titleLineEdit)
        self.footerLabel = QtGui.QLabel(self.optionsWidget)
        self.footerLabel.setObjectName(u'footerLabel')
        self.optionsLayout.addWidget(self.footerLabel)
        self.footerTextEdit = SpellTextEdit(self.optionsWidget, False)
        self.footerTextEdit.setObjectName(u'footerTextEdit')
        self.optionsLayout.addWidget(self.footerTextEdit)
        self.optionsGroupBox = QtGui.QGroupBox()
        self.groupLayout = QtGui.QVBoxLayout()
        self.slideTextCheckBox = QtGui.QCheckBox()
        self.groupLayout.addWidget(self.slideTextCheckBox)
        self.pageBreakAfterText = QtGui.QCheckBox()
        self.groupLayout.addWidget(self.pageBreakAfterText)
        self.notesCheckBox = QtGui.QCheckBox()
        self.groupLayout.addWidget(self.notesCheckBox)
        self.metaDataCheckBox = QtGui.QCheckBox()
        self.groupLayout.addWidget(self.metaDataCheckBox)
        self.groupLayout.addStretch(1)
        self.optionsGroupBox.setLayout(self.groupLayout)
        self.optionsLayout.addWidget(self.optionsGroupBox)

        self.retranslateUi(printServiceDialog)
        QtCore.QObject.connect(self.optionsButton,
            QtCore.SIGNAL(u'toggled(bool)'), self.toggleOptions)

    def retranslateUi(self, printServiceDialog):
        printServiceDialog.setWindowTitle(UiStrings().PrintService)
        self.zoomOutButton.setToolTip(translate('OpenLP.PrintServiceForm',
            'Zoom Out'))
        self.zoomOriginalButton.setToolTip(translate('OpenLP.PrintServiceForm',
            'Zoom Original'))
        self.zoomInButton.setToolTip(translate('OpenLP.PrintServiceForm',
            'Zoom In'))
        self.optionsButton.setText(translate('OpenLP.PrintServiceForm',
            'Options'))
        self.titleLabel.setText(translate('OpenLP.PrintServiceForm', 'Title:'))
        self.footerLabel.setText(translate('OpenLP.PrintServiceForm',
            'Custom Footer Text:'))
        self.optionsGroupBox.setTitle(
            translate('OpenLP.PrintServiceForm','Other Options'))
        self.slideTextCheckBox.setText(translate('OpenLP.PrintServiceForm',
            'Include slide text if available'))
        self.pageBreakAfterText.setText(translate('OpenLP.PrintServiceForm',
            'Add page break before each text item'))
        self.notesCheckBox.setText(translate('OpenLP.PrintServiceForm',
            'Include service item notes'))
        self.metaDataCheckBox.setText(translate('OpenLP.PrintServiceForm',
            'Include play length of media items'))
        self.titleLineEdit.setText(translate('OpenLP.PrintServiceForm',
            'Service Sheet'))
        # Do not change the order.
        self.zoomComboBox.addItems([
            translate('OpenLP.PrintServiceDialog', 'Fit Page'),
            translate('OpenLP.PrintServiceDialog', 'Fit Width'),
            u'100%',
            u'75%',
            u'50%',
            u'25%']
        )




# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

"""
The :mod:`core` module provides all core application functions

All the core functions of the OpenLP application including the GUI, settings,
logging and a plugin framework are contained within the openlp.core module.
"""

import os
import sys
import platform
import logging
from optparse import OptionParser
from traceback import format_exception

from PyQt4 import QtCore, QtGui

from openlp.core.lib import Receiver, check_directory_exists
from openlp.core.lib.settings import Settings
from openlp.core.lib.ui import UiStrings
from openlp.core.resources import qInitResources
from openlp.core.ui.mainwindow import MainWindow
from openlp.core.ui.firsttimelanguageform import FirstTimeLanguageForm
from openlp.core.ui.firsttimeform import FirstTimeForm
from openlp.core.ui.exceptionform import ExceptionForm
from openlp.core.ui import SplashScreen, ScreenList
from openlp.core.utils import AppLocation, LanguageManager, VersionThread, \
    get_application_version


__all__ = [u'OpenLP', u'main']


log = logging.getLogger()
nt_repair_stylesheet = u"""
QMainWindow::separator
{
  border: none;
}

QDockWidget::title
{
  border: 1px solid palette(dark);
  padding-left: 5px;
  padding-top: 2px;
  margin: 1px 0;
}

QToolBar
{
  border: none;
  margin: 0;
  padding: 0;
}
"""


class OpenLP(QtGui.QApplication):
    """
    The core application class. This class inherits from Qt's QApplication
    class in order to provide the core of the application.
    """

    args = []

    def exec_(self):
        """
        Override exec method to allow the shared memory to be released on exit
        """
        self.eventLoopIsActive = True
        QtGui.QApplication.exec_()
        self.sharedMemory.detach()

    def run(self, args, testing=False):
        """
        Run the OpenLP application.
        """
        self.eventLoopIsActive = False
        # On Windows, the args passed into the constructor are ignored. Not
        # very handy, so set the ones we want to use. On Linux and FreeBSD, in
        # order to set the WM_CLASS property for X11, we pass "OpenLP" in as a
        # command line argument. This interferes with files being passed in as
        # command line arguments, so we remove it from the list.
        if 'OpenLP' in args:
            args.remove('OpenLP')
        self.args.extend(args)
        # provide a listener for widgets to reqest a screen update.
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'openlp_process_events'), self.processEvents)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'cursor_busy'), self.setBusyCursor)
        QtCore.QObject.connect(Receiver.get_receiver(),
            QtCore.SIGNAL(u'cursor_normal'), self.setNormalCursor)
        # Decide how many screens we have and their size
        screens = ScreenList.create(self.desktop())
        # First time checks in settings
        has_run_wizard = Settings().value(
            u'general/has run wizard', QtCore.QVariant(False)).toBool()
        if not has_run_wizard:
            if FirstTimeForm(screens).exec_() == QtGui.QDialog.Accepted:
                Settings().setValue(u'general/has run wizard',
                    QtCore.QVariant(True))
        # Correct stylesheet bugs
        if os.name == u'nt':
            base_color = self.palette().color(QtGui.QPalette.Active,
                QtGui.QPalette.Base)
            application_stylesheet = \
                u'QTableWidget, QListWidget, QTreeWidget ' + \
                u'{alternate-background-color: ' + \
                base_color.name() + ';}\n'
            application_stylesheet += nt_repair_stylesheet
            self.setStyleSheet(application_stylesheet)
        # show the splashscreen
        show_splash = Settings().value(
            u'general/show splash', QtCore.QVariant(True)).toBool()
        if show_splash:
            self.splash = SplashScreen()
            self.splash.show()
        # make sure Qt really display the splash screen
        self.processEvents()
        # start the main app window
        self.mainWindow = MainWindow(self)
        self.mainWindow.show()
        if show_splash:
            # now kill the splashscreen
            self.splash.finish(self.mainWindow)
            log.debug(u'Splashscreen closed')
        # make sure Qt really display the splash screen
        self.processEvents()
        self.mainWindow.repaint()
        self.processEvents()
        if not has_run_wizard:
            self.mainWindow.firstTime()
        update_check = Settings().value(
            u'general/update check', QtCore.QVariant(True)).toBool()
        if update_check:
            VersionThread(self.mainWindow).start()
        Receiver.send_message(u'live_display_blank_check')
        self.mainWindow.appStartup()
        # Skip exec_() for gui tests
        if not testing:
            return self.exec_()

    def isAlreadyRunning(self):
        """
        Look to see if OpenLP is already running and ask if a 2nd copy
        is to be started.
        """
        self.sharedMemory = QtCore.QSharedMemory('OpenLP')
        if self.sharedMemory.attach():
            status = QtGui.QMessageBox.critical(None,
                UiStrings().Error, UiStrings().OpenLPStart,
                QtGui.QMessageBox.StandardButtons(
                QtGui.QMessageBox.Yes | QtGui.QMessageBox.No))
            if status == QtGui.QMessageBox.No:
                return True
            return False
        else:
            self.sharedMemory.create(1)
            return False

    def hookException(self, exctype, value, traceback):
        if not hasattr(self, u'mainWindow'):
            log.exception(''.join(format_exception(exctype, value, traceback)))
            return
        if not hasattr(self, u'exceptionForm'):
            self.exceptionForm = ExceptionForm(self.mainWindow)
        self.exceptionForm.exceptionTextEdit.setPlainText(
            ''.join(format_exception(exctype, value, traceback)))
        self.setNormalCursor()
        self.exceptionForm.exec_()

    def setBusyCursor(self):
        """
        Sets the Busy Cursor for the Application
        """
        self.setOverrideCursor(QtCore.Qt.BusyCursor)
        self.processEvents()

    def setNormalCursor(self):
        """
        Sets the Normal Cursor for the Application
        """
        self.restoreOverrideCursor()

    def event(self, event):
        """
        Enables direct file opening on OS X
        """
        if event.type() == QtCore.QEvent.FileOpen:
            file_name = event.file()
            log.debug(u'Got open file event for %s!', file_name)
            self.args.insert(0, unicode(file_name))
            return True
        # Mac OS X should restore app window when user clicked on the OpenLP icon
        # in the Dock bar. However, OpenLP constists of multiple windows and this
        # does not work. This workaround fixes that.
        # The main OpenLP window is restored when it was previously minimized.
        elif event.type() == QtCore.QEvent.ApplicationActivate:
            if sys.platform.startswith('darwin') and self.mainWindow.isMinimized():
                self.mainWindow.showNormal()
                return True
        return QtGui.QApplication.event(self, event)


def set_up_logging(log_path):
    """
    Setup our logging using log_path
    """
    check_directory_exists(log_path, True)
    filename = os.path.join(log_path, u'openlp.log')
    logfile = logging.FileHandler(filename, u'w')
    logfile.setFormatter(logging.Formatter(
        u'%(asctime)s %(name)-55s %(levelname)-8s %(message)s'))
    log.addHandler(logfile)
    if log.isEnabledFor(logging.DEBUG):
        print 'Logging to:', filename


def main(args=None):
    """
    The main function which parses command line options and then runs
    the PyQt4 Application.
    """
    # Set up command line options.
    usage = 'Usage: %prog [options] [qt-options]'
    parser = OptionParser(usage=usage)
    parser.add_option('-e', '--no-error-form', dest='no_error_form',
        action='store_true', help='Disable the error notification form.')
    parser.add_option('-l', '--log-level', dest='loglevel',
        default='warning', metavar='LEVEL', help='Set logging to LEVEL '
        'level. Valid values are "debug", "info", "warning".')
    parser.add_option('-p', '--portable', dest='portable',
        action='store_true', help='Specify if this should be run as a '
        'portable app, off a USB flash drive (not implemented).')
    parser.add_option('-d', '--dev-version', dest='dev_version',
        action='store_true', help='Ignore the version file and pull the '
        'version directly from Bazaar')
    parser.add_option('-s', '--style', dest='style',
        help='Set the Qt4 style (passed directly to Qt4).')
    parser.add_option('--testing', dest='testing',
        action='store_true', help='Run by testing framework')
    # Parse command line options and deal with them.
    # Use args supplied programatically if possible.
    (options, args) = parser.parse_args(args) if args else parser.parse_args()
    qt_args = []
    if options.loglevel.lower() in ['d', 'debug']:
        log.setLevel(logging.DEBUG)
    elif options.loglevel.lower() in ['w', 'warning']:
        log.setLevel(logging.WARNING)
    else:
        log.setLevel(logging.INFO)
    if options.style:
        qt_args.extend(['-style', options.style])
    # Throw the rest of the arguments at Qt, just in case.
    qt_args.extend(args)
    # Bug #1018855: Set the WM_CLASS property in X11
    if platform.system() not in ['Windows', 'Darwin']:
        qt_args.append('OpenLP')
    # Initialise the resources
    qInitResources()
    # Now create and actually run the application.
    app = OpenLP(qt_args)
    app.setOrganizationName(u'OpenLP')
    app.setOrganizationDomain(u'openlp.org')
    if options.portable:
        app.setApplicationName(u'OpenLPPortable')
        Settings.setDefaultFormat(Settings.IniFormat)
        # Get location OpenLPPortable.ini
        app_path = AppLocation.get_directory(AppLocation.AppDir)
        set_up_logging(os.path.abspath(os.path.join(app_path, u'..',
            u'..', u'Other')))
        log.info(u'Running portable')
        portable_settings_file = os.path.abspath(os.path.join(app_path, u'..',
            u'..', u'Data', u'OpenLP.ini'))
        # Make this our settings file
        log.info(u'INI file: %s', portable_settings_file)
        Settings.setFilename(portable_settings_file)
        portable_settings = Settings()
        # Set our data path
        data_path = os.path.abspath(os.path.join(app_path,
            u'..', u'..', u'Data',))
        log.info(u'Data path: %s', data_path)
        # Point to our data path
        portable_settings.setValue(u'advanced/data path', data_path)
        portable_settings.setValue(u'advanced/is portable', True)
        portable_settings.sync()
    else:
        set_up_logging(AppLocation.get_directory(AppLocation.CacheDir))
        app.setApplicationName(u'OpenLP')
    app.setApplicationVersion(get_application_version()[u'version'])
    # Instance check
    if not options.testing:
        # Instance check
        if app.isAlreadyRunning():
            sys.exit()
    # First time checks in settings
    if not Settings().value(u'general/has run wizard',
        QtCore.QVariant(False)).toBool():
        if not FirstTimeLanguageForm().exec_():
            # if cancel then stop processing
            sys.exit()
    # i18n Set Language
    language = LanguageManager.get_language()
    app_translator, default_translator = \
        LanguageManager.get_translator(language)
    if not app_translator.isEmpty():
        app.installTranslator(app_translator)
    if not default_translator.isEmpty():
        app.installTranslator(default_translator)
    else:
        log.debug(u'Could not find default_translator.')
    if not options.no_error_form:
        sys.excepthook = app.hookException
    # Do not run method app.exec_() when running gui tests
    if options.testing:
        app.run(qt_args, testing=True)
        # For gui tests we need access to window intances and their components
        return app
    else:
        sys.exit(app.run(qt_args))



# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2014 Raoul Snyman                                        #
# Portions copyright (c) 2008-2014 Tim Bentley, Gerald Britton, Jonathan      #
# Corwin, Samuel Findlay, Michael Gorven, Scott Guerrieri, Matthias Hub,      #
# Meinert Jordan, Armin Köhler, Erik Lundin, Edwin Lunando, Brian T. Meyer.   #
# Joshua Miller, Stevan Pettit, Andreas Preikschat, Mattias Põldaru,          #
# Christian Richter, Philip Ridout, Simon Scudder, Jeffrey Smith,             #
# Maikel Stuivenberg, Martin Thompson, Jon Tibble, Dave Warnock,              #
# Frode Woldsund, Martin Zibricky                                             #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

import os
import sys

from PyQt4 import QtGui, QtCore
from ctypes import *
from ctypes.wintypes import RECT

class PPTViewer(QtGui.QWidget):
    """
    Standalone Test Harness for the pptviewlib library
    """
    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)
        self.pptid = -1
        self.setWindowTitle(u'PowerPoint Viewer Test')

        ppt_label = QtGui.QLabel(u'Open PowerPoint file')
        slide_label = QtGui.QLabel(u'Go to slide #')
        self.pptEdit = QtGui.QLineEdit()
        self.slideEdit = QtGui.QLineEdit()
        x_label = QtGui.QLabel(u'X pos')
        y_label = QtGui.QLabel(u'Y pos')
        width_label = QtGui.QLabel(u'Width')
        height_label = QtGui.QLabel(u'Height')
        self.xEdit = QtGui.QLineEdit(u'100')
        self.yEdit = QtGui.QLineEdit(u'100')
        self.widthEdit = QtGui.QLineEdit(u'900')
        self.heightEdit = QtGui.QLineEdit(u'700')
        self.total = QtGui.QLabel()
        ppt_btn = QtGui.QPushButton(u'Open')
        ppt_dlg_btn = QtGui.QPushButton(u'...')
        folder_label = QtGui.QLabel(u'Slide .bmp path')
        self.folderEdit = QtGui.QLineEdit(u'slide')
        slide_btn = QtGui.QPushButton(u'Go')
        prev = QtGui.QPushButton(u'Prev')
        next = QtGui.QPushButton(u'Next')
        blank = QtGui.QPushButton(u'Blank')
        unblank = QtGui.QPushButton(u'Unblank')
        restart = QtGui.QPushButton(u'Restart')
        close = QtGui.QPushButton(u'Close')
        resume = QtGui.QPushButton(u'Resume')
        stop = QtGui.QPushButton(u'Stop')
        grid = QtGui.QGridLayout()
        row = 0
        grid.addWidget(folder_label, 0, 0)
        grid.addWidget(self.folderEdit, 0, 1)
        row = row + 1
        grid.addWidget(x_label, row, 0)
        grid.addWidget(self.xEdit, row, 1)
        grid.addWidget(y_label, row, 2)
        grid.addWidget(self.yEdit, row, 3)
        row = row + 1
        grid.addWidget(width_label, row, 0)
        grid.addWidget(self.widthEdit, row, 1)
        grid.addWidget(height_label, row, 2)
        grid.addWidget(self.heightEdit, row, 3)
        row = row + 1
        grid.addWidget(ppt_label, row, 0)
        grid.addWidget(self.pptEdit, row, 1)
        grid.addWidget(ppt_dlg_btn, row, 2)
        grid.addWidget(ppt_btn, row, 3)
        row = row + 1
        grid.addWidget(slide_label, row, 0)
        grid.addWidget(self.slideEdit, row, 1)
        grid.addWidget(slide_btn, row, 2)
        row = row + 1
        grid.addWidget(prev, row, 0)
        grid.addWidget(next, row, 1)
        row = row + 1
        grid.addWidget(blank, row, 0)
        grid.addWidget(unblank, row, 1)
        row = row + 1
        grid.addWidget(restart, row, 0)
        grid.addWidget(close, row, 1)
        row = row + 1
        grid.addWidget(stop, row, 0)
        grid.addWidget(resume, row, 1)
        self.connect(ppt_btn, QtCore.SIGNAL(u'clicked()'), self.openClick)
        self.connect(ppt_dlg_btn, QtCore.SIGNAL(u'clicked()'), self.openDialog)
        self.connect(slide_btn, QtCore.SIGNAL(u'clicked()'), self.gotoClick)
        self.connect(prev, QtCore.SIGNAL(u'clicked()'), self.prevClick)
        self.connect(next, QtCore.SIGNAL(u'clicked()'), self.nextClick)
        self.connect(blank, QtCore.SIGNAL(u'clicked()'), self.blankClick)
        self.connect(unblank, QtCore.SIGNAL(u'clicked()'), self.unblankClick)
        self.connect(restart, QtCore.SIGNAL(u'clicked()'), self.restartClick)
        self.connect(close, QtCore.SIGNAL(u'clicked()'), self.closeClick)
        self.connect(stop, QtCore.SIGNAL(u'clicked()'), self.stopClick)
        self.connect(resume, QtCore.SIGNAL(u'clicked()'), self.resumeClick)
        self.setLayout(grid)
        self.resize(300, 150)

    def prevClick(self):
        if self.pptid < 0:
            return
        self.pptdll.PrevStep(self.pptid)
        self.updateCurrSlide()
        app.processEvents()

    def nextClick(self):
        if self.pptid < 0:
            return
        self.pptdll.NextStep(self.pptid)
        self.updateCurrSlide()
        app.processEvents()

    def blankClick(self):
        if self.pptid < 0:
            return
        self.pptdll.Blank(self.pptid)
        app.processEvents()

    def unblankClick(self):
        if self.pptid < 0:
            return
        self.pptdll.Unblank(self.pptid)
        app.processEvents()

    def restartClick(self):
        if self.pptid < 0:
            return
        self.pptdll.RestartShow(self.pptid)
        self.updateCurrSlide()
        app.processEvents()

    def stopClick(self):
        if self.pptid < 0:
            return
        self.pptdll.Stop(self.pptid)
        app.processEvents()

    def resumeClick(self):
        if self.pptid < 0:
            return
        self.pptdll.Resume(self.pptid)
        app.processEvents()

    def closeClick(self):
        if self.pptid < 0:
            return
        self.pptdll.ClosePPT(self.pptid)
        self.pptid = -1
        app.processEvents()

    def openClick(self):
        oldid = self.pptid;
        rect = RECT(int(self.xEdit.text()), int(self.yEdit.text()),
            int(self.widthEdit.text()), int(self.heightEdit.text()))
        filename = os.path.normpath(unicode(self.pptEdit.text())).encode('utf-16-le') + '\0'
        folder = os.path.normpath(unicode(self.folderEdit.text())).encode('utf-16-le') + '\0'
        self.pptid = self.pptdll.OpenPPT(filename, None, rect, folder)
        print u'id: ' + unicode(self.pptid)
        if oldid >= 0:
            self.pptdll.ClosePPT(oldid);
        slides = self.pptdll.GetSlideCount(self.pptid)
        print u'slidecount: ' + unicode(slides)
        self.total.setNum(self.pptdll.GetSlideCount(self.pptid))
        self.updateCurrSlide()

    def updateCurrSlide(self):
        if self.pptid < 0:
            return
        slide = unicode(self.pptdll.GetCurrentSlide(self.pptid))
        print u'currslide: ' + slide
        self.slideEdit.setText(slide)
        app.processEvents()

    def gotoClick(self):
        if self.pptid < 0:
            return
        print self.slideEdit.text()
        self.pptdll.GotoSlide(self.pptid, int(self.slideEdit.text()))
        self.updateCurrSlide()
        app.processEvents()

    def openDialog(self):
        self.pptEdit.setText(QtGui.QFileDialog.getOpenFileName(self,
            u'Open file'))

if __name__ == '__main__':
    pptdll = cdll.LoadLibrary(r'pptviewlib.dll')
    pptdll.SetDebug(1)
    print u'Begin...'
    app = QtGui.QApplication(sys.argv)
    window = PPTViewer()
    window.pptdll = pptdll
    window.show()
    sys.exit(app.exec_())
